@@ -31,11 +31,12 @@
 #include "cdbase.h"
 #include "debug.h"
 
-int FreeBSDCDInit(const char *);
-int FreeBSDCDDeInit(void);
-long FreeBSDCDReadTOC(unsigned long *);
-int FreeBSDCDGetStatus(void);
-int FreeBSDCDReadSectorFAD(unsigned long, void *);
+static int FreeBSDCDInit(const char *);
+static void FreeBSDCDDeInit(void);
+static s32 FreeBSDCDReadTOC(u32 *);
+static int FreeBSDCDGetStatus(void);
+static int FreeBSDCDReadSectorFAD(u32, void *);
+static void FreeBSDCDReadAheadFAD(u32);
 
 CDInterface ArchCD = {
 	CDCORE_ARCH,
@@ -44,12 +45,13 @@ CDInterface ArchCD = {
 	FreeBSDCDDeInit,
 	FreeBSDCDGetStatus,
 	FreeBSDCDReadTOC,
-	FreeBSDCDReadSectorFAD
+	FreeBSDCDReadSectorFAD,
+	FreeBSDCDReadAheadFAD,
 };
 
-int hCDROM;
+static int hCDROM;
 
-int FreeBSDCDInit(const char * cdrom_name) {
+static int FreeBSDCDInit(const char * cdrom_name) {
 	int bsize = 2352;
 
 	if ((hCDROM = open(cdrom_name, O_RDONLY | O_NONBLOCK)) == -1) {
@@ -65,19 +67,16 @@ int FreeBSDCDInit(const char * cdrom_name) {
 	return 0;
 }
 
-int FreeBSDCDDeInit(void) {
-	if (hCDROM == -1) {
-		return -1;
+static void FreeBSDCDDeInit(void) {
+	if (hCDROM != -1) {
+		close(hCDROM);
 	}
-	close(hCDROM);
 
 	LOG("CDDeInit OK\n");
-
-	return 0;
 }
 
 
-long FreeBSDCDReadTOC(unsigned long * TOC)
+static s32 FreeBSDCDReadTOC(u32 * TOC)
 {
    int success;
    struct ioc_toc_header ctTOC;
@@ -139,7 +138,7 @@ long FreeBSDCDReadTOC(unsigned long * TOC)
    return 0;
 }
 
-int FreeBSDCDGetStatus(void) {
+static int FreeBSDCDGetStatus(void) {
 	// 0 - CD Present, disc spinning
 	// 1 - CD Present, disc not spinning
 	// 2 - CD not present
@@ -149,7 +148,7 @@ int FreeBSDCDGetStatus(void) {
 	return 0;
 }
 
-int FreeBSDCDReadSectorFAD(unsigned long FAD, void *buffer) {
+static int FreeBSDCDReadSectorFAD(u32 FAD, void *buffer) {
 	if (hCDROM != -1)
 	{
 		lseek(hCDROM, (FAD - 150) * 2352, SEEK_SET);
@@ -160,3 +159,8 @@ int FreeBSDCDReadSectorFAD(unsigned long FAD, void *buffer) {
 
 	return 0;
 }
+
+static void FreeBSDCDReadAheadFAD(UNUSED u32 FAD)
+{
+	// No-op
+}
