@@ -2,6 +2,7 @@
     Copyright 2004-2005 Theo Berkau
     Copyright 2005 Weston Yager
     Copyright 2006-2008 Guillaume Duhamel
+    Copyright 2010 Lawrence Sebald
 
     This file is part of Yabause.
 
@@ -38,11 +39,12 @@
 
 #include "cdbase.h"
 
-int MacOSXCDInit(const char *);
-int MacOSXCDDeInit();
-int MacOSXCDGetStatus();
-long MacOSXCDReadTOC(unsigned long *);
-int MacOSXCDReadSectorFAD(unsigned long, void *);
+static int MacOSXCDInit(const char *);
+static void MacOSXCDDeInit(void);
+static int MacOSXCDGetStatus(void);
+static s32 MacOSXCDReadTOC(u32 *);
+static int MacOSXCDReadSectorFAD(u32, void *);
+static void MacOSXCDReadAheadFAD(u32);
 
 CDInterface ArchCD = {
 CDCORE_ARCH,
@@ -51,12 +53,13 @@ MacOSXCDInit,
 MacOSXCDDeInit,
 MacOSXCDGetStatus,
 MacOSXCDReadTOC,
-MacOSXCDReadSectorFAD
+MacOSXCDReadSectorFAD,
+MacOSXCDReadAheadFAD,
 };
 
-int hCDROM;
+static int hCDROM;
 
-int MacOSXCDInit(const char * useless_for_now)
+static int MacOSXCDInit(const char * useless_for_now)
 {
 	CFMutableDictionaryRef  classesToMatch;
 	io_iterator_t mediaIterator;
@@ -103,17 +106,15 @@ int MacOSXCDInit(const char * useless_for_now)
 	return 0;
 }
 
-int MacOSXCDDeInit() 
+static void MacOSXCDDeInit(void) 
 {
 	if (hCDROM != -1) 
 	{
 		close(hCDROM);
 	}
-	
-	return 0;
 }
 
-CDTOC * GetTOCFromCDPath()
+static CDTOC * GetTOCFromCDPath(void)
 {
 	CFMutableDictionaryRef  classesToMatch;
 	io_iterator_t mediaIterator;
@@ -140,7 +141,7 @@ CDTOC * GetTOCFromCDPath()
 	return TOC;
 }
 
-long MacOSXCDReadTOC(unsigned long *TOC) 
+static s32 MacOSXCDReadTOC(u32 *TOC) 
 {
   	int add150 = 150, tracks = 0;
 	u_char track;
@@ -172,7 +173,7 @@ long MacOSXCDReadTOC(unsigned long *TOC)
 	return (0xCC * 2);
 }
 
-int MacOSXCDGetStatus(void) 
+static int MacOSXCDGetStatus(void) 
 {
 	// 0 - CD Present, disc spinning
 	// 1 - CD Present, disc not spinning
@@ -184,16 +185,43 @@ int MacOSXCDGetStatus(void)
 	return 0;
 }
 
-int MacOSXCDReadSectorFAD(unsigned long FAD, void *buffer) 
+static int MacOSXCDReadSectorFAD(u32 FAD, void *buffer) 
 {
-	int blockSize = 2352;
+	const int blockSize = 2352;
+#ifdef CRAB_REWRITE
+    const int cacheBlocks = 32;
+    static u8 cache[blockSize * cacheBlocks];
+    static u32 cacheFAD = 0xFFFFFF00;
+#endif
 	
 	if (hCDROM != -1) 
 	{
+#ifdef CRAB_REWRITE
+        /* See if the block we are looking for is in the cache already... */
+        if(FAD < cacheFAD || FAD >= cacheFAD + cacheBlocks) {
+            /* Cache miss, read some blocks from the cd, then we'll hit the
+               cache below. */
+            if(!pread(hCDROM, cache, blockSize * cacheBlocks,
+                      (FAD - 150) * blockSize)) {
+                return 0;
+            }
+
+            cacheFAD = FAD;
+        }
+
+        /* Cache hit, copy the block out. */
+        memcpy(buffer, cache + (blockSize * (FAD - cacheFAD)), blockSize);
+        return 1;
+#else
 		if (pread(hCDROM, buffer, blockSize, (FAD - 150) * blockSize))
 			return true;
+#endif
 	}
 	
 	return false;
 }
 
+static void MacOSXCDReadAheadFAD(UNUSED u32 FAD)
+{
+	// No-op
+}
