@@ -35,10 +35,11 @@ typedef struct
         int id;
         const char *Name;
         int (*Init)(const char *);
-        int (*DeInit)();
-        int (*GetStatus)();
+        void (*DeInit)(void);
+        int (*GetStatus)(void);
         s32 (*ReadTOC)(u32 *TOC);
         int (*ReadSectorFAD)(u32 FAD, void *buffer);
+        void (*ReadAheadFAD)(u32 FAD);
 } CDInterface;
 
 extern CDInterface DummyCD;
