@@ -1,4 +1,4 @@
-/*  Copyright 2010 Lawrence Sebald
+/*  Copyright 2010, 2012 Lawrence Sebald
 
     This file is part of Yabause.
 
@@ -39,7 +39,7 @@
 YabauseController *controller;
 
 @interface YabauseController (InternalFunctions)
-- (void)startEmulationWithCDCore:(int)cdcore;
+- (void)startEmulationWithCDCore:(int)cdcore CDPath:(const char *)fn;
 - (void)emulationThread:(id)ignored;
 - (void)terminateEmulation;
 @end
@@ -115,12 +115,25 @@ static void FlipToggle(NSMenuItem *item) {
 {
     /* This will simply start up the system with the dummy CD core, so there's
        no way it'll actually read that there's a disc to be played. */
-    [self startEmulationWithCDCore:CDCORE_DUMMY];
+    [self startEmulationWithCDCore:CDCORE_DUMMY CDPath:NULL];
 }
 
 - (IBAction)runCD:(id)sender
 {
-    [self startEmulationWithCDCore:CDCORE_ARCH];
+    [self startEmulationWithCDCore:CDCORE_ARCH CDPath:NULL];
+}
+
+- (IBAction)runISO:(id)sender
+{
+    NSOpenPanel *p = [NSOpenPanel openPanel];
+    NSArray *types = [NSArray arrayWithObjects:@"iso", @"cue", nil];
+
+    [p setAllowedFileTypes:types];
+    if([p runModal] == NSFileHandlingPanelOKButton) {
+        NSString *fn = [[[p URLs] objectAtIndex:0] path];
+        [self startEmulationWithCDCore:CDCORE_ISO
+                                CDPath:[fn fileSystemRepresentation]];
+    }
 }
 
 - (IBAction)toggleFullscreen:(id)sender
@@ -195,14 +208,14 @@ static void FlipToggle(NSMenuItem *item) {
 
             /* Mute the audio before we actually pause otherwise the user might
                not like the result... */
-            ScspMuteAudio();
+            ScspMuteAudio(SCSP_MUTE_SYSTEM);
             [_runLock lock];
             [sender setState:NSOnState];
         }
         else {
             _paused = NO;
             [_runLock unlock];
-            ScspUnMuteAudio();
+            ScspUnMuteAudio(SCSP_MUTE_SYSTEM);
             [sender setState:NSOffState];
         }
     }
@@ -225,10 +238,11 @@ static void FlipToggle(NSMenuItem *item) {
 
 @implementation YabauseController (InternalFunctions)
 
-- (void)startEmulationWithCDCore:(int)cdcore
+- (void)startEmulationWithCDCore:(int)cdcore CDPath:(const char *)fn
 {
     if(!_running) {
         yabauseinit_struct yinit;
+        int initok;
         NSString *bios = [prefs biosPath];
         NSString *mpeg = [prefs mpegPath];
         NSString *bram = [prefs bramPath];
@@ -244,7 +258,7 @@ static void FlipToggle(NSMenuItem *item) {
         yinit.regionid = [prefs region];
         yinit.biospath = ([bios length] > 0 && ![prefs emulateBios]) ?
             [bios UTF8String] : NULL;
-        yinit.cdpath = NULL;
+        yinit.cdpath = fn;
         yinit.buppath = NULL;
         yinit.mpegpath = ([mpeg length] > 0) ? [mpeg UTF8String] : NULL;
         yinit.videoformattype = ([prefs region] < 10) ? VIDEOFORMATTYPE_NTSC :
@@ -257,12 +271,12 @@ static void FlipToggle(NSMenuItem *item) {
         /* Set up the internal save ram if specified. */
         if([bram length] > 0) {
             const char *tmp = [bram UTF8String];
-
-            _bramFile = (char *)malloc(strlen(tmp) + 1);
-            strcpy(_bramFile, tmp);
-            yinit.buppath = _bramFile;
+            yinit.buppath = _bramFile = strdup(tmp);
         }
 
+        if(fn)
+            _isoFile = strdup(fn);
+
         /* Set up the cartridge stuff based on what was selected. */
         if(yinit.carttype == CART_NETLINK) {
             yinit.cartpath = NULL;
@@ -281,8 +295,14 @@ static void FlipToggle(NSMenuItem *item) {
             return;
         }
 
-        YabauseInit(&yinit);
-        YabauseSetDecilineMode(0);
+        [[view openGLContext] makeCurrentContext];
+        initok = YabauseInit(&yinit);
+        [NSOpenGLContext clearCurrentContext];
+        if (initok != 0) {
+            return;
+        }
+
+        YabauseSetDecilineMode(1);
 
         _running = YES;
         _doneExecuting = NO;
@@ -308,7 +328,7 @@ static void FlipToggle(NSMenuItem *item) {
        drawing to nothingness. */
     [[view openGLContext] makeCurrentContext];
 
-    ScspUnMuteAudio();
+    ScspUnMuteAudio(SCSP_MUTE_SYSTEM);
 
     while(_running) {
         /* If we get paused from the GUI, we'll end up waiting in this lock
@@ -329,7 +349,7 @@ static void FlipToggle(NSMenuItem *item) {
         [_runLock unlock];
     }
 
-    ScspMuteAudio();
+    ScspMuteAudio(SCSP_MUTE_SYSTEM);
 
     _doneExecuting = YES;
     [pool release];
@@ -347,10 +367,10 @@ static void FlipToggle(NSMenuItem *item) {
 
         YabauseDeInit();
 
-        if(_bramFile) {
-            free(_bramFile);
-            _bramFile = NULL;
-        }
+        free(_bramFile);
+        _bramFile = NULL;
+        free(_isoFile);
+        _isoFile = NULL;
 
         _emuThd = nil;
     }
