@@ -38,7 +38,6 @@
 #include "yui.h"
 
 #include <stdlib.h>
-#include <stdarg.h>
 #include <limits.h>
 
 #if defined WORDS_BIGENDIAN
@@ -96,7 +95,6 @@ void FASTCALL VIDGCDVdp2SetPriorityNBG1(int priority);
 void FASTCALL VIDGCDVdp2SetPriorityNBG2(int priority);
 void FASTCALL VIDGCDVdp2SetPriorityNBG3(int priority);
 void FASTCALL VIDGCDVdp2SetPriorityRBG0(int priority);
-void VIDGCDOnScreenDebugMessage(char *string, ...);
 void VIDGCDGetGlSize(int *width, int *height);
 void VIDGCDVdp1SwapFrameBuffer(void);
 void VIDGCDVdp1EraseFrameBuffer(void);
@@ -128,13 +126,6 @@ VIDGCDVdp2Reset,
 VIDGCDVdp2DrawStart,
 VIDGCDVdp2DrawEnd,
 VIDGCDVdp2DrawScreens,
-VIDGCDVdp2SetResolution,
-VIDGCDVdp2SetPriorityNBG0,
-VIDGCDVdp2SetPriorityNBG1,
-VIDGCDVdp2SetPriorityNBG2,
-VIDGCDVdp2SetPriorityNBG3,
-VIDGCDVdp2SetPriorityRBG0,
-VIDGCDOnScreenDebugMessage,
 VIDGCDGetGlSize,
 };
 
@@ -166,9 +157,6 @@ static int outputheight;
 static int resxratio;
 static int resyratio;
 
-static char message[512];
-static int msglength;
-
 typedef struct { s16 x; s16 y; } vdp1vertex;
 
 typedef struct
@@ -1554,20 +1542,6 @@ int VIDGCDInit(void)
    vdp2height = 224;
 
 #ifdef USE_OPENGL
-   YuiSetVideoAttribute(DOUBLEBUFFER, 1);
-
-   if (!YglScreenInit(8, 8, 8, 24))
-   {
-      if (!YglScreenInit(4, 4, 4, 24))
-      {
-         if (!YglScreenInit(5, 6, 5, 16))
-         {
-	    YuiErrorMsg("Couldn't set GL mode\n");
-            return -1;
-         }
-      }
-   }
-
    glClear(GL_COLOR_BUFFER_BIT);
 
    glMatrixMode(GL_PROJECTION);
@@ -1579,7 +1553,6 @@ int VIDGCDInit(void)
    glOrtho(-320, 320, -224, 224, 1, 0);
    outputwidth = 320;
    outputheight = 224;
-   msglength = 0;
 #endif
 
    return 0;
@@ -1614,11 +1587,6 @@ void VIDGCDResize(unsigned int w, unsigned int h, int on)
 #ifdef USE_OPENGL
    IsFullscreen = on;
 
-   if (on)
-      YuiSetVideoMode(w, h, 32, 1);
-   else
-      YuiSetVideoMode(w, h, 32, 0);
-
    glClear(GL_COLOR_BUFFER_BIT);
 
    glMatrixMode(GL_PROJECTION);
@@ -3079,6 +3047,13 @@ void VIDGCDVdp2DrawScreens(void)
 {
    int i;
 
+   VIDGCDVdp2SetResolution(Vdp2Regs->TVMD);
+   VIDGCDVdp2SetPriorityNBG0(Vdp2Regs->PRINA & 0x7);
+   VIDGCDVdp2SetPriorityNBG1((Vdp2Regs->PRINA >> 8) & 0x7);
+   VIDGCDVdp2SetPriorityNBG2(Vdp2Regs->PRINB & 0x7);
+   VIDGCDVdp2SetPriorityNBG3((Vdp2Regs->PRINB >> 8) & 0x7);
+   VIDGCDVdp2SetPriorityRBG0(Vdp2Regs->PRIR & 0x7);
+
     /* Draw all the screens, from the lowest priority one forward. */
     for(i = 1; i < 8; ++i)
     {
@@ -3209,18 +3184,6 @@ void FASTCALL VIDGCDVdp2SetPriorityRBG0(int priority)
 
 //////////////////////////////////////////////////////////////////////////////
 
-void VIDGCDOnScreenDebugMessage(char *string, ...)
-{
-   va_list arglist;
-
-   va_start(arglist, string);
-   vsprintf(message, string, arglist);
-   va_end(arglist);
-   msglength = strlen(message);
-}
-
-//////////////////////////////////////////////////////////////////////////////
-
 void VIDGCDGetScreenSize(int *width, int *height)
 {
    *width = vdp2width;
