@@ -43,7 +43,6 @@
 #include "../m68kcore.h"
 #include "../m68kc68k.h"
 #include "pergtk.h"
-#include "../psp/psp-sh2.h"
 
 #include "settings.h"
 
@@ -559,13 +558,6 @@ int main(int argc, char *argv[]) {
 	return 0;
 }
 
-void YuiSetVideoAttribute(int type, int val) {
-}
-
-int YuiSetVideoMode(int width, int height, int bpp, int fullscreen) {
-	return 0;
-}
-
 void YuiSwapBuffers(void) {
 	yui_window_update(YUI_WINDOW(yui));
 }
