@@ -46,16 +46,17 @@ int M68KInit(int coreid) {
    return 0;
 }
 
-void M68KDummyInit(void) {
+static int M68KDummyInit(void) {
+	return 0;
 }
 
-void M68KDummyDeInit(void) {
+static void M68KDummyDeInit(void) {
 }
 
-void M68KDummyReset(void) {
+static void M68KDummyReset(void) {
 }
 
-s32 FASTCALL M68KDummyExec(UNUSED s32 cycle) {
+static s32 FASTCALL M68KDummyExec(UNUSED s32 cycle) {
 	T2WriteWord(SoundRam, 0x700, 0);
 	T2WriteWord(SoundRam, 0x710, 0);
 	T2WriteWord(SoundRam, 0x720, 0);
@@ -70,67 +71,70 @@ s32 FASTCALL M68KDummyExec(UNUSED s32 cycle) {
 	return 0;
 }
 
-u32 M68KDummyGetDReg(UNUSED u32 num) {
+static void M68KDummySync(void) {
+}
+
+static u32 M68KDummyGetDReg(UNUSED u32 num) {
 	return 0;
 }
 
-u32 M68KDummyGetAReg(UNUSED u32 num) {
+static u32 M68KDummyGetAReg(UNUSED u32 num) {
 	return 0;
 }
 
-u32 M68KDummyGetPC(void) {
+static u32 M68KDummyGetPC(void) {
 	return 0;
 }
 
-u32 M68KDummyGetSR(void) {
+static u32 M68KDummyGetSR(void) {
 	return 0;
 }
 
-u32 M68KDummyGetUSP(void) {
+static u32 M68KDummyGetUSP(void) {
 	return 0;
 }
 
-u32 M68KDummyGetMSP(void) {
+static u32 M68KDummyGetMSP(void) {
 	return 0;
 }
 
-void M68KDummySetDReg(UNUSED u32 num, UNUSED u32 val) {
+static void M68KDummySetDReg(UNUSED u32 num, UNUSED u32 val) {
 }
 
-void M68KDummySetAReg(UNUSED u32 num, UNUSED u32 val) {
+static void M68KDummySetAReg(UNUSED u32 num, UNUSED u32 val) {
 }
 
-void M68KDummySetPC(UNUSED u32 val) {
+static void M68KDummySetPC(UNUSED u32 val) {
 }
 
-void M68KDummySetSR(UNUSED u32 val) {
+static void M68KDummySetSR(UNUSED u32 val) {
 }
 
-void M68KDummySetUSP(UNUSED u32 val) {
+static void M68KDummySetUSP(UNUSED u32 val) {
 }
 
-void M68KDummySetMSP(UNUSED u32 val) {
+static void M68KDummySetMSP(UNUSED u32 val) {
 }
 
-void M68KDummySetFetch(UNUSED u32 low_adr, UNUSED u32 high_adr, UNUSED pointer fetch_adr) {
+static void M68KDummySetFetch(UNUSED u32 low_adr, UNUSED u32 high_adr, UNUSED pointer fetch_adr) {
 }
 
-void FASTCALL M68KDummySetIRQ(UNUSED s32 level) {
+static void FASTCALL M68KDummySetIRQ(UNUSED s32 level) {
 }
 
-void FASTCALL M68KDummyTouchMem(u32 address) {
+static void FASTCALL M68KDummyWriteNotify(u32 address, u32 size) {
 }
 
-void M68KDummySetReadB(UNUSED M68K_READ *Func) {
+static void M68KDummySetReadB(UNUSED M68K_READ *Func) {
 }
 
-void M68KDummySetReadW(UNUSED M68K_READ *Func) {
+static void M68KDummySetReadW(UNUSED M68K_READ *Func) {
 }
 
-void M68KDummySetWriteB(UNUSED M68K_WRITE *Func) {
+static void M68KDummySetWriteB(UNUSED M68K_WRITE *Func) {
 }
 
-void M68KDummySetWriteW(UNUSED M68K_WRITE *Func) {
+static void M68KDummySetWriteW(UNUSED M68K_WRITE *Func) {
 }
 
 M68K_struct M68KDummy = {
@@ -140,6 +144,7 @@ M68K_struct M68KDummy = {
 	M68KDummyDeInit,
 	M68KDummyReset,
 	M68KDummyExec,
+	M68KDummySync,
 	M68KDummyGetDReg,
 	M68KDummyGetAReg,
 	M68KDummyGetPC,
@@ -154,7 +159,7 @@ M68K_struct M68KDummy = {
 	M68KDummySetMSP,
 	M68KDummySetFetch,
 	M68KDummySetIRQ,
-	M68KDummyTouchMem,
+	M68KDummyWriteNotify,
 	M68KDummySetReadB,
 	M68KDummySetReadW,
 	M68KDummySetWriteB,
