@@ -31,6 +31,7 @@
 
 #define SDL_MAX_AXIS_VALUE 0x110000
 #define SDL_MIN_AXIS_VALUE 0x100000
+#define SDL_HAT_VALUE 0x200000
 #define SDL_MEDIUM_AXIS_VALUE (int)(32768 / 2)
 #define SDL_BUTTON_PRESSED 1
 #define SDL_BUTTON_RELEASED 0
@@ -38,9 +39,8 @@
 int PERSDLJoyInit(void);
 void PERSDLJoyDeInit(void);
 int PERSDLJoyHandleEvents(void);
-void PERSDLJoyNothing(void);
 
-u32 PERSDLJoyScan(void);
+u32 PERSDLJoyScan(u32 flags);
 void PERSDLJoyFlush(void);
 void PERSDLKeyName(u32 key, char * name, int size);
 
@@ -50,23 +50,23 @@ PERCORE_SDLJOY,
 PERSDLJoyInit,
 PERSDLJoyDeInit,
 PERSDLJoyHandleEvents,
-PERSDLJoyNothing,
 PERSDLJoyScan,
 1,
-PERSDLJoyFlush
-#ifdef PERKEYNAME
-,PERSDLKeyName
-#endif
+PERSDLJoyFlush,
+PERSDLKeyName
 };
 
 typedef struct {
 	SDL_Joystick* mJoystick;
 	s16* mScanStatus;
+	Uint8* mHatStatus;
 } PERSDLJoystick;
 
 unsigned int SDL_PERCORE_INITIALIZED = 0;
 unsigned int SDL_PERCORE_JOYSTICKS_INITIALIZED = 0;
 PERSDLJoystick* SDL_PERCORE_JOYSTICKS = 0;
+unsigned int SDL_HAT_VALUES[] = { SDL_HAT_UP, SDL_HAT_RIGHT, SDL_HAT_LEFT, SDL_HAT_DOWN };
+const unsigned int SDL_HAT_VALUES_NUM = sizeof(SDL_HAT_VALUES) / sizeof(SDL_HAT_VALUES[0]);
 
 //////////////////////////////////////////////////////////////////////////////
 
@@ -78,7 +78,11 @@ int PERSDLJoyInit(void) {
 	{
 		return 0;
 	}
-	
+
+#if defined (_MSC_VER) && SDL_VERSION_ATLEAST(2,0,0)
+   SDL_SetMainReady();
+#endif
+
 	// init joysticks
 	if ( SDL_InitSubSystem( SDL_INIT_JOYSTICK ) == -1 )
 	{
@@ -99,6 +103,7 @@ int PERSDLJoyInit(void) {
 		
 		SDL_PERCORE_JOYSTICKS[ i ].mJoystick = joy;
 		SDL_PERCORE_JOYSTICKS[ i ].mScanStatus = joy ? malloc(sizeof(s16) * SDL_JoystickNumAxes( joy )) : 0;
+		SDL_PERCORE_JOYSTICKS[ i ].mHatStatus = joy ? malloc(sizeof(Uint8) * SDL_JoystickNumHats( joy )) : 0;
 		
 		if ( joy )
 		{
@@ -106,6 +111,10 @@ int PERSDLJoyInit(void) {
 			{
 				SDL_PERCORE_JOYSTICKS[ i ].mScanStatus[ j ] = SDL_JoystickGetAxis( joy, j );
 			}
+			for ( j = 0; j < SDL_JoystickNumHats( joy ); j++ )
+			{
+				SDL_PERCORE_JOYSTICKS[ i ].mHatStatus[ j ] = SDL_JoystickGetHat( joy, j );
+			}
 		}
 	}
 	
@@ -123,11 +132,17 @@ void PERSDLJoyDeInit(void) {
 		int i;
 		for ( i = 0; i < SDL_PERCORE_JOYSTICKS_INITIALIZED; i++ )
 		{
+#if SDL_VERSION_ATLEAST(2,0,0)
+         if ( SDL_PERCORE_JOYSTICKS[ i ].mJoystick )
+#else
 			if ( SDL_JoystickOpened( i ) )
-			{
-				SDL_JoystickClose( SDL_PERCORE_JOYSTICKS[ i ].mJoystick );
-				free( SDL_PERCORE_JOYSTICKS[ i ].mScanStatus );
-			}
+#endif
+         {
+            SDL_JoystickClose( SDL_PERCORE_JOYSTICKS[ i ].mJoystick );
+
+            free( SDL_PERCORE_JOYSTICKS[ i ].mScanStatus );
+            free( SDL_PERCORE_JOYSTICKS[ i ].mHatStatus );
+         }
 		}
 		free( SDL_PERCORE_JOYSTICKS );
 	}
@@ -141,17 +156,16 @@ void PERSDLJoyDeInit(void) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void PERSDLJoyNothing(void) {
-}
-
-//////////////////////////////////////////////////////////////////////////////
-
 int PERSDLJoyHandleEvents(void) {
 	int joyId;
 	int i;
+	int j;
 	SDL_Joystick* joy;
 	Sint16 cur;
 	Uint8 buttonState;
+	Uint8 newHatState;
+	Uint8 oldHatState;
+	int hatValue;
 	
 	// update joysticks states
 	SDL_JoystickUpdate();
@@ -170,6 +184,8 @@ int PERSDLJoyHandleEvents(void) {
 		for ( i = 0; i < SDL_JoystickNumAxes( joy ); i++ )
 		{
 			cur = SDL_JoystickGetAxis( joy, i );
+
+			PerAxisValue((joyId << 18) | SDL_MEDIUM_AXIS_VALUE | i, (u8)(((int)cur+32768) >> 8));
 			
 			if ( cur < -SDL_MEDIUM_AXIS_VALUE )
 			{
@@ -202,6 +218,32 @@ int PERSDLJoyHandleEvents(void) {
 				PerKeyUp( (joyId << 18) | (i +1) );
 			}
 		}
+
+		// check hats
+		for ( i = 0; i < SDL_JoystickNumHats( joy ); i++ )
+		{
+			newHatState = SDL_JoystickGetHat( joy, i );
+			oldHatState = SDL_PERCORE_JOYSTICKS[ joyId ].mHatStatus[ i ];
+
+			for ( j = 0 ; j < SDL_HAT_VALUES_NUM; j++ )
+			{
+				hatValue = SDL_HAT_VALUES[ j ];
+				if ( oldHatState & hatValue && ~newHatState & hatValue )
+				{
+					PerKeyUp( (joyId << 18) | SDL_HAT_VALUE | (hatValue << 4) | i );
+				}
+			}
+			for ( j = 0 ; j < SDL_HAT_VALUES_NUM; j++ )
+			{
+				hatValue = SDL_HAT_VALUES[ j ];
+				if ( ~oldHatState & hatValue && newHatState & hatValue )
+				{
+					PerKeyDown( (joyId << 18) | SDL_HAT_VALUE | (hatValue << 4) | i);
+				}
+			}
+
+			SDL_PERCORE_JOYSTICKS[ joyId ].mHatStatus[ i ] = newHatState;
+		}
 	}
 	
 	// execute yabause
@@ -216,12 +258,13 @@ int PERSDLJoyHandleEvents(void) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-u32 PERSDLJoyScan( void ) {
+u32 PERSDLJoyScan( u32 flags ) {
 	// init vars
 	int joyId;
 	int i;
 	SDL_Joystick* joy;
 	Sint16 cur;
+	Uint8 hatState;
 	
 	// update joysticks states
 	SDL_JoystickUpdate();
@@ -245,22 +288,51 @@ u32 PERSDLJoyScan( void ) {
 			{
 				if ( cur < -SDL_MEDIUM_AXIS_VALUE )
 				{
-					return (joyId << 18) | SDL_MIN_AXIS_VALUE | i;
+					if (flags & PERSF_AXIS)
+						return (joyId << 18) | SDL_MEDIUM_AXIS_VALUE | i;
+					if (flags & PERSF_HAT)
+						return (joyId << 18) | SDL_MIN_AXIS_VALUE | i;
 				}
 				else if ( cur > SDL_MEDIUM_AXIS_VALUE )
 				{
-					return (joyId << 18) | SDL_MAX_AXIS_VALUE | i;
+					if (flags & PERSF_AXIS)
+						return (joyId << 18) | SDL_MEDIUM_AXIS_VALUE | i;
+					if (flags & PERSF_HAT)
+						return (joyId << 18) | SDL_MAX_AXIS_VALUE | i;
 				}
 			}
 		}
 
-		// check buttons
-		for ( i = 0; i < SDL_JoystickNumButtons( joy ); i++ )
+		if (flags & PERSF_BUTTON)
 		{
-			if ( SDL_JoystickGetButton( joy, i ) == SDL_BUTTON_PRESSED )
+			// check buttons
+			for ( i = 0; i < SDL_JoystickNumButtons( joy ); i++ )
 			{
-				return (joyId << 18) | (i +1);
-				break;
+				if ( SDL_JoystickGetButton( joy, i ) == SDL_BUTTON_PRESSED )
+				{
+					return (joyId << 18) | (i +1);
+					break;
+				}
+			}
+		}
+
+		if (flags & PERSF_HAT)
+		{
+			// check hats
+			for ( i = 0; i < SDL_JoystickNumHats( joy ); i++ )
+			{
+				hatState = SDL_JoystickGetHat( joy, i );
+				switch (hatState)
+				{
+					case SDL_HAT_UP:
+					case SDL_HAT_RIGHT:
+					case SDL_HAT_DOWN:
+					case SDL_HAT_LEFT:
+						return (joyId << 18) | SDL_HAT_VALUE | (hatState << 4) | i;
+						break;
+					default:
+						break;
+				}
 			}
 		}
 	}
