@@ -1,5 +1,5 @@
 /*  Copyright 2005 Guillaume Duhamel
-	Copyright 2005-2006 Theo Berkau
+	Copyright 2005-2006, 2013 Theo Berkau
 	Copyright 2008 Filipe Azevedo <pasnox@gmail.com>
 
 	This file is part of Yabause.
@@ -28,6 +28,7 @@
 #include <QGroupBox>
 #include <QTreeWidget>
 #include <QPointer>
+#include <QDir>
 
 // cores
 
@@ -88,6 +89,9 @@ SoundInterface_struct *SNDCoreList[] = {
 #ifdef HAVE_DIRECTSOUND
 &SNDDIRECTX,
 #endif
+#ifdef ARCH_IS_MACOSX
+&SNDMac,
+#endif
 NULL
 };
 
@@ -121,17 +125,15 @@ QPointer<VolatileSettings> mVolatileSettings = 0;
 // ports padbits
 QMap<uint, PerPad_struct*> mPort1PadsBits;
 QMap<uint, PerPad_struct*> mPort2PadsBits;
+QMap<uint, PerMouse_struct*> mPort1MouseBits;
+QMap<uint, PerMouse_struct*> mPort2MouseBits;
+QMap<uint, PerAnalog_struct*> mPort1AnalogBits;
+QMap<uint, PerAnalog_struct*> mPort2AnalogBits;
 
 extern "C" 
 {
 	void YuiErrorMsg(const char *string)
 	{ QtYabause::mainWindow()->appendLog( string ); }
-
-	int YuiSetVideoMode(int /*width*/, int /*height*/, int /*bpp*/, int /*fullscreen*/)
-	{ return 0; }
-
-	void YuiSetVideoAttribute(int /*type*/, int /*val*/)
-	{}
 	
 	void YuiSwapBuffers()
 	{ QtYabause::mainWindow()->swapBuffers(); }
@@ -165,12 +167,42 @@ VolatileSettings* QtYabause::volatileSettings( bool create )
 	return mVolatileSettings;
 }
 
+QList <translation_struct> QtYabause::getTranslationList()
+{
+	QList <translation_struct> translations;
+#ifdef HAVE_LIBMINI18N
+	QDir transDir=QDir(YTSDIR, "*.yts");	
+
+	foreach(QString file, transDir.entryList())
+	{
+		if (!file.endsWith(".yts"))
+			continue;
+		translation_struct trans;
+		trans.file = QString(YTSDIR) + QString("/") + file;
+
+		// Let's get it down just to language code
+		QStringList string = file.remove(".yts").split("_");
+		if (file.startsWith("yabause_"))
+			string.removeFirst();
+		QString localeStr = string.join("_");
+		// Find the locale
+		QLocale locale = QLocale(localeStr);	
+		// Now we should be good for the language name
+		trans.name = locale.nativeLanguageName();
+		translations.append(trans);
+	}
+#endif
+	return translations;
+}
+
 int QtYabause::setTranslationFile()
 {
 #ifdef HAVE_LIBMINI18N
 	const QString s = settings()->value( "General/Translation" ).toString();
 	if ( ! s.isEmpty() )
 	{
+		if (s == "#") return 0; // magic value for "no translation" (aka english)
+
 		const char* filePath = qstrdup( s.toLocal8Bit().constData() );
 		if ( mini18n_set_locale( filePath ) == 0 )
 		{
@@ -194,16 +226,17 @@ int QtYabause::setTranslationFile()
 
 int QtYabause::logTranslation()
 {
-	return 0;
 #ifdef HAVE_LIBMINI18N
-	const QString s = settings()->value( "General/Translation" ).toString().replace( ".yts", "_log.yts" );
-	if ( s.isEmpty() )
-		return 0;
-	const char* filePath = qstrdup( s.toLocal8Bit().constData() );
-	return mini18n_set_log( filePath );
-#else
-	return 0;
+	if (settings()->value( "General/LogUntranslated", false ).toBool())
+	{
+		const QString s = settings()->value( "General/Translation" ).toString().replace( ".yts", "_log.yts" );
+		if ( s.isEmpty() )
+			return 0;
+		const char* filePath = qstrdup( s.toLocal8Bit().constData() );
+		return mini18n_set_log( filePath );
+	}
 #endif
+	return 0;
 }
 
 void QtYabause::closeTranslation()
@@ -393,16 +426,18 @@ VideoInterface_struct QtYabause::defaultVIDCore()
 
 OSD_struct QtYabause::defaultOSDCore()
 {
-#if defined HAVE_LIBGLUT
-	return OSDGlut;
-#else
 	return OSDSoft;
-#endif
 }
 
 PerInterface_struct QtYabause::defaultPERCore()
 {
+#ifdef HAVE_LIBSDL
+	return PERSDLJoy;
+#elif defined (HAVE_DIRECTINPUT)
+	return PERDIRECTX;
+#else
 	return PERQT;
+#endif
 }
 
 SH2Interface_struct QtYabause::defaultSH2Core()
@@ -431,3 +466,47 @@ void QtYabause::clearPadsBits()
 	mPort1PadsBits.clear();
 	mPort2PadsBits.clear();
 }
+
+QMap<uint, PerAnalog_struct*>* QtYabause::portAnalogBits( uint portNumber )
+{
+   switch ( portNumber )
+   {
+   case 1:
+      return &mPort1AnalogBits;
+      break;
+   case 2:
+      return &mPort2AnalogBits;
+      break;
+   default:
+      return 0;
+      break;
+   }
+}
+
+void QtYabause::clear3DAnalogBits()
+{
+   mPort1AnalogBits.clear();
+   mPort2AnalogBits.clear();
+}
+
+QMap<uint, PerMouse_struct*>* QtYabause::portMouseBits( uint portNumber )
+{
+   switch ( portNumber )
+   {
+   case 1:
+      return &mPort1MouseBits;
+      break;
+   case 2:
+      return &mPort2MouseBits;
+      break;
+   default:
+      return 0;
+      break;
+   }
+}
+
+void QtYabause::clearMouseBits()
+{
+   mPort1MouseBits.clear();
+   mPort2MouseBits.clear();
+}
