@@ -1,5 +1,5 @@
 /*  Copyright 2005 Guillaume Duhamel
-	Copyright 2005-2006 Theo Berkau
+	Copyright 2005-2006, 2013 Theo Berkau
 	Copyright 2008 Filipe Azevedo <pasnox@gmail.com>
 
 	This file is part of Yabause.
@@ -23,6 +23,9 @@
 
 extern "C"
 {
+#ifdef Q_OS_WIN
+#include <qt_windows.h>
+#endif
 	#include "../yabause.h"
 	#include "../peripheral.h"
 	#include "../sh2core.h"
@@ -60,9 +63,9 @@ extern "C"
 	#include "../vdp2debug.h"
 
 	#include "PerQt.h"
-#ifdef Q_OS_WIN
-	#include <qt_windows.h>
-	#include "../windows/cd.h"
+
+#ifdef ARCH_IS_MACOSX
+    #include "../sndmac.h"
 #endif
 }
 
@@ -74,11 +77,18 @@ class Settings;
 class VolatileSettings;
 class QWidget;
 
+typedef struct 
+{
+	QString file;
+	QString name;
+} translation_struct;
+
 namespace QtYabause
 {
 	UIYabause* mainWindow( bool create = true );
 	Settings* settings( bool create = true );
 	VolatileSettings* volatileSettings( bool create = true );
+	QList <translation_struct> getTranslationList();
 	int setTranslationFile();
 	int logTranslation();
 	void closeTranslation();
@@ -109,6 +119,10 @@ namespace QtYabause
 	// padsbits
 	QMap<uint, PerPad_struct*>* portPadsBits( uint portNumber );
 	void clearPadsBits();
+	QMap<uint, PerAnalog_struct*>* portAnalogBits( uint portNumber );
+	void clear3DAnalogBits();
+	QMap<uint, PerMouse_struct*>* portMouseBits( uint portNumber );
+	void clearMouseBits();
 };
 
 #endif // QTYABAUSE_H
