@@ -1,5 +1,5 @@
 /*  Copyright 2005 Guillaume Duhamel
-	Copyright 2005-2006 Theo Berkau
+	Copyright 2005-2006, 2013 Theo Berkau
 	Copyright 2008 Filipe Azevedo <pasnox@gmail.com>
 
 	This file is part of Yabause.
@@ -26,6 +26,7 @@
 
 #include "../peripheral.h"
 
+#include <QDateTime>
 #include <QStringList>
 #include <QDebug>
 
@@ -51,7 +52,7 @@ void YabauseThread::initEmulation()
 {
 	reloadSettings();
 	mInit = YabauseInit( &mYabauseConf );
-   SetOSDToggle(showFPS);
+	SetOSDToggle(showFPS);
 }
 
 void YabauseThread::deInitEmulation()
@@ -102,7 +103,7 @@ bool YabauseThread::pauseEmulation( bool pause, bool reset )
 				vs->value("autostart/binary/address").toUInt());
 		}
 		else if (vs->value("autostart/load").toBool()) {
-			YabLoadStateSlot( QtYabause::volatileSettings()->value( "General/SaveStates", getDataDirPath() ).toString().toAscii().constData(), vs->value("autostart/load/slot").toInt() );
+			YabLoadStateSlot( QtYabause::volatileSettings()->value( "General/SaveStates", getDataDirPath() ).toString().toLatin1().constData(), vs->value("autostart/load/slot").toInt() );
 		}
 		vs->setValue("autostart", false);
 	}
@@ -162,8 +163,51 @@ void YabauseThread::reloadControllers()
 					}
 					break;
 				}
+				case PERWHEEL:
+					QtYabause::mainWindow()->appendLog( "Wheel controller type is not yet supported" );
+					break;
+				case PER3DPAD:
+				{
+					PerAnalog_struct* analogbits = Per3DPadAdd( port == 1 ? &PORTDATA1 : &PORTDATA2 );
+
+					settings->beginGroup( QString( "Input/Port/%1/Id/%2/Controller/%3/Key" ).arg( port ).arg( id ).arg( type ) );
+					QStringList analogKeys = settings->childKeys();
+					settings->endGroup();
+
+					analogKeys.sort();
+					foreach ( const QString& analogKey, analogKeys )
+					{
+						const QString key = settings->value( QString( UIPortManager::mSettingsKey ).arg( port ).arg( id ).arg( type ).arg( analogKey ) ).toString();
+
+						PerSetKey( key.toUInt(), analogKey.toUInt(), analogbits );
+					}
+					break;
+				}
+				case PERGUN:
+					QtYabause::mainWindow()->appendLog( "Gun controller type is not yet supported" );
+					break;
+				case PERKEYBOARD:
+					QtYabause::mainWindow()->appendLog( "Keyboard controller type is not yet supported" );
+					break;
 				case PERMOUSE:
-					QtYabause::mainWindow()->appendLog( "Mouse controller type is not yet supported" );
+				{
+					PerMouse_struct* mousebits = PerMouseAdd( port == 1 ? &PORTDATA1 : &PORTDATA2 );
+
+					settings->beginGroup( QString( "Input/Port/%1/Id/%2/Controller/%3/Key" ).arg( port ).arg( id ).arg( type ) );
+					QStringList mouseKeys = settings->childKeys();
+					settings->endGroup();
+
+					mouseKeys.sort();
+					foreach ( const QString& mouseKey, mouseKeys )
+					{
+						const QString key = settings->value( QString( UIPortManager::mSettingsKey ).arg( port ).arg( id ).arg( type ).arg( mouseKey ) ).toString();
+
+						PerSetKey( key.toUInt(), mouseKey.toUInt(), mousebits );
+					}
+					break;
+				}
+				case 0:
+					// Unconnected
 					break;
 				default:
 					QtYabause::mainWindow()->appendLog( "Invalid controller type" );
@@ -173,6 +217,46 @@ void YabauseThread::reloadControllers()
 	}
 }
 
+static struct tm *localtime_qt(const QDateTime &input, struct tm *result)
+{
+	QDate date(input.date());
+	result->tm_year = date.year() - 1900;
+	result->tm_mon = date.month();
+	result->tm_mday = date.day();
+	result->tm_wday = date.dayOfWeek();
+	result->tm_yday = date.dayOfYear();
+
+	QTime time(input.time());
+	result->tm_sec = time.second();
+	result->tm_min = time.minute();
+	result->tm_hour = time.hour();
+
+	return result;
+}
+
+void YabauseThread::reloadClock()
+{
+	QString tmp;
+	Settings* s = QtYabause::settings();
+
+	if (mYabauseConf.basetime == 0)
+		tmp = "";
+	else
+		tmp = QDateTime::fromTime_t(mYabauseConf.basetime).toString();
+
+	// Clock sync
+	mYabauseConf.clocksync = (int)s->value( "General/ClockSync", mYabauseConf.clocksync ).toBool();
+	tmp = s->value( "General/FixedBaseTime", tmp ).toString();
+	if (!tmp.isEmpty() && mYabauseConf.clocksync) 
+	{
+		QDateTime date = QDateTime::fromString(tmp, Qt::ISODate);
+		mYabauseConf.basetime = (long)date.toTime_t();	
+	}
+	else {
+		mYabauseConf.basetime = 0;
+	}
+}
+
 void YabauseThread::reloadSettings()
 {
 	//QMutexLocker l( &mMutex );
@@ -196,7 +280,7 @@ void YabauseThread::reloadSettings()
 		mYabauseConf.regionid = 0;
 	else
 	{
-		switch ( r[0].toAscii() )
+		switch ( r[0].toLatin1() )
 		{
 			case 'J': mYabauseConf.regionid = 1; break;
 			case 'T': mYabauseConf.regionid = 2; break;
@@ -208,17 +292,19 @@ void YabauseThread::reloadSettings()
 			case 'L': mYabauseConf.regionid = 0xD; break;
 		}
 	}
-	mYabauseConf.biospath = strdup( vs->value( "General/Bios", mYabauseConf.biospath ).toString().toAscii().constData() );
-	mYabauseConf.cdpath = strdup( vs->value( "General/CdRomISO", mYabauseConf.cdpath ).toString().toAscii().constData() );   
+	mYabauseConf.biospath = strdup( vs->value( "General/Bios", mYabauseConf.biospath ).toString().toLatin1().constData() );
+	mYabauseConf.cdpath = strdup( vs->value( "General/CdRomISO", mYabauseConf.cdpath ).toString().toLatin1().constData() ); 
    showFPS = vs->value( "General/ShowFPS", false ).toBool();
-	mYabauseConf.buppath = strdup( vs->value( "Memory/Path", mYabauseConf.buppath ).toString().toAscii().constData() );
-	mYabauseConf.mpegpath = strdup( vs->value( "MpegROM/Path", mYabauseConf.mpegpath ).toString().toAscii().constData() );
-	mYabauseConf.cartpath = strdup( vs->value( "Cartridge/Path", mYabauseConf.cartpath ).toString().toAscii().constData() );
+	mYabauseConf.buppath = strdup( vs->value( "Memory/Path", mYabauseConf.buppath ).toString().toLatin1().constData() );
+	mYabauseConf.mpegpath = strdup( vs->value( "MpegROM/Path", mYabauseConf.mpegpath ).toString().toLatin1().constData() );
+	mYabauseConf.cartpath = strdup( vs->value( "Cartridge/Path", mYabauseConf.cartpath ).toString().toLatin1().constData() );
 	mYabauseConf.videoformattype = vs->value( "Video/VideoFormat", mYabauseConf.videoformattype ).toInt();
 	
-	emit requestSize( QSize( vs->value( "Video/Width", 0 ).toInt(), vs->value( "Video/Height", 0 ).toInt() ) );
+	emit requestSize( QSize( vs->value( "Video/WinWidth", 0 ).toInt(), vs->value( "Video/WinHeight", 0 ).toInt() ) );
 	emit requestFullscreen( vs->value( "Video/Fullscreen", false ).toBool() );
+	emit requestVolumeChange( vs->value( "Sound/Volume", 100 ).toInt() );
 
+	reloadClock();
 	reloadControllers();
 }
 
