@@ -23,6 +23,9 @@
 #include "QtYabause.h"
 #include "Settings.h"
 #include "ui/UIYabause.h"
+#ifndef NO_CLI
+#include "Arguments.h"
+#endif
 
 int main( int argc, char** argv )
 {
@@ -32,9 +35,14 @@ int main( int argc, char** argv )
 	app.setApplicationName( QString( "Qt Yabause v%1 - A Beautiful And Under-rated Saturn Emulator" ).arg( VERSION ) );
 	// init settings
 	Settings::setIniInformations();
+#ifdef HAVE_LIBMINI18N
 	// set translation file
 	if ( QtYabause::setTranslationFile() == -1 )
 		qWarning( "Can't set translation file" );
+#endif
+#ifndef NO_CLI
+	Arguments::parse();
+#endif
 	// show main window
 	QtYabause::mainWindow()->setWindowTitle( app.applicationName() );
 	QtYabause::mainWindow()->show();
