@@ -1,4 +1,4 @@
-/*	Copyright 2012 Theo Berkau <cwx@cyberwarriorx.com>
+/*	Copyright 2012-2013 Theo Berkau <cwx@cyberwarriorx.com>
 
 	This file is part of Yabause.
 
@@ -31,6 +31,9 @@ public:
 	UIDebugSH2(bool master, YabauseThread *mYabauseThread, QWidget* parent = 0 );
    void updateRegList();
    void updateCodeList(u32 addr);
+   void updateBackTrace();
+   void updateTrackInfLoop();
+   void updateAll();
    u32 getRegister(int index, int *size);
    void setRegister(int index, u32 value);
    bool addCodeBreakpoint(u32 addr);
@@ -38,6 +41,10 @@ public:
    bool addMemoryBreakpoint(u32 addr, u32 flags);
    bool delMemoryBreakpoint(u32 addr);
    void stepInto();
+   void stepOver();
+   void stepOut();
+   void reserved1();
+   void reserved2();
 protected:
 
 protected slots:
