@@ -1,5 +1,5 @@
 /*  Copyright 2005 Guillaume Duhamel
-	Copyright 2005-2006 Theo Berkau
+	Copyright 2005-2006, 2013 Theo Berkau
 	Copyright 2008 Filipe Azevedo <pasnox@gmail.com>
 
 	This file is part of Yabause.
@@ -26,6 +26,7 @@
 
 #include <QDir>
 #include <QList>
+#include <QDesktopWidget>
 
 extern "C" {
 extern M68K_struct* M68KCoreList[];
@@ -69,17 +70,25 @@ const Items mCartridgeTypes = Items()
 	<< Item( "6", "8 Mbit Dram" )
 	<< Item( "7", "32 Mbit Dram" )
 	<< Item( "8", "Netlink" )
-	<< Item( "9", "16 Mbit ROM" );
+	<< Item( "9", "16 Mbit ROM" )
+   << Item( "10", "Japanese Modem" );
 
-const Items mVideoFromats = Items()
+const Items mVideoFormats = Items()
 	<< Item( "0", "NTSC" )
 	<< Item( "1", "PAL" );
 
-UISettings::UISettings( QWidget* p )
+UISettings::UISettings( QList <supportedRes_struct> *supportedResolutions, QList <translation_struct> *translations, QWidget* p )
 	: QDialog( p )
 {
 	// setup dialog
 	setupUi( this );
+
+	QRect maxWinRect = QApplication::desktop()->availableGeometry();
+	
+	
+	leWinWidth->setValidator(new QIntValidator(0, maxWinRect.width(), leWinWidth));
+	leWinHeight->setValidator(new QIntValidator(0, maxWinRect.height(), leWinHeight));
+
 	pmPort1->setPort( 1 );
 	pmPort1->loadSettings();
 	pmPort2->setPort( 2 );
@@ -90,9 +99,24 @@ UISettings::UISettings( QWidget* p )
 		setWindowFlags( Qt::Sheet );
 	}
 
+	setupCdDrives();
+
 	// load cores informations
 	loadCores();
 
+	supportedRes = *supportedResolutions;
+	loadSupportedResolutions();
+
+#ifdef HAVE_LIBMINI18N
+	trans = *translations;
+	loadTranslations();
+#else
+   lTranslation->hide();
+	cbTranslation->hide();
+#endif
+
+	loadShortcuts();
+
 	// load settings
 	loadSettings();
 	
@@ -127,6 +151,44 @@ void UISettings::requestFolder( const QString& c, QLineEdit* e )
 		e->setText( s );
 }
 
+QStringList getCdDriveList()
+{
+	QStringList list;
+
+#if defined Q_OS_WIN
+	foreach( QFileInfo drive, QDir::drives () )
+	{
+		LPCWSTR driveString = (LPCWSTR)drive.filePath().utf16();
+		if (GetDriveTypeW(driveString) == DRIVE_CDROM)
+			list.append(drive.filePath());
+	}
+#elif defined Q_OS_LINUX
+	FILE * f = fopen("/proc/sys/dev/cdrom/info", "r");
+	char buffer[1024];
+	char drive_name[10];
+	char drive_path[255];
+
+	if (f != NULL) {
+		while (fgets(buffer, 1024, f) != NULL) {
+			if (sscanf(buffer, "drive name:%s", drive_name) == 1) {
+				sprintf(drive_path, "/dev/%s", drive_name);
+
+				list.append(drive_path);
+			}
+		}
+	}
+#elif defined Q_OS_MAC
+#endif
+	return list;
+}
+
+void UISettings::setupCdDrives()
+{
+	QStringList list=getCdDriveList();
+	foreach(QString string, list)
+		cbCdDrive->addItem(string);
+}
+
 void UISettings::tbBrowse_clicked()
 {
 	// get toolbutton sender
@@ -142,14 +204,12 @@ void UISettings::tbBrowse_clicked()
 			return;
 		}
 		else if ( cbCdRom->currentText().contains( "iso", Qt::CaseInsensitive ) )
-			requestFile( QtYabause::translate( "Select your iso/cue/bin file" ), leCdRom, QtYabause::translate( "CD Images (*.iso *.cue *.bin)" ) );
+			requestFile( QtYabause::translate( "Select your iso/cue/bin file" ), leCdRom, QtYabause::translate( "CD Images (*.iso *.cue *.bin *.mds)" ) );
 		else
 			requestFolder( QtYabause::translate( "Choose a cdrom drive/mount point" ), leCdRom );
 	}
 	else if ( tb == tbSaveStates )
 		requestFolder( QtYabause::translate( "Choose a folder to store save states" ), leSaveStates );
-	else if ( tb == tbTranslation )
-		requestFile( QtYabause::translate( "Choose the translation file to use" ), leTranslation, QtYabause::translate( "Yabause Translation Files (*.yts)" ) );
 	else if ( tb == tbCartridge )
 		requestNewFile( QtYabause::translate( "Choose a cartridge file" ), leCartridge );
 	else if ( tb == tbMemory )
@@ -169,6 +229,38 @@ void UISettings::on_cbInput_currentIndexChanged( int id )
 	pmPort2->setCore( core );
 }
 
+void UISettings::on_cbCdRom_currentIndexChanged( int id )
+{
+	CDInterface* core = QtYabause::getCDCore( cbCdRom->itemData( id ).toInt() );
+
+	Q_ASSERT( core );
+
+	switch (core->id)
+	{
+		case CDCORE_DUMMY:
+			cbCdDrive->setVisible(false);
+			leCdRom->setVisible(false);
+			tbCdRom->setVisible(false);
+			break;
+		case CDCORE_ISO:
+			cbCdDrive->setVisible(false);
+			leCdRom->setVisible(true);
+			tbCdRom->setVisible(true);
+			break;
+		case CDCORE_ARCH:
+			cbCdDrive->setVisible(true);
+			leCdRom->setVisible(false);
+			tbCdRom->setVisible(false);
+			break;
+		default: break;
+	}
+}
+
+void UISettings::on_cbClockSync_stateChanged( int state )
+{
+	dteBaseTime->setVisible( state == Qt::Checked );
+}
+
 void UISettings::loadCores()
 {
 	// CD Drivers
@@ -189,7 +281,7 @@ void UISettings::loadCores()
 #endif
 	
 	// Video Formats
-	foreach ( const Item& it, mVideoFromats )
+	foreach ( const Item& it, mVideoFormats )
 		cbVideoFormat->addItem( QtYabause::translate( it.Name ), it.id );
 	
 	// SND Drivers
@@ -213,6 +305,71 @@ void UISettings::loadCores()
 		cbSH2Interpreter->addItem( QtYabause::translate( SH2CoreList[i]->Name ), SH2CoreList[i]->id );
 }
 
+void UISettings::loadSupportedResolutions()
+{
+	if (supportedRes.count() == 0)
+	{
+		lFullScreenResolution->setVisible(false);
+		cbFullscreenResolution->setVisible(false);
+		return;
+	}
+
+	// Load supported screen resolutions
+	for (int i = 0; i < this->supportedRes.count(); i++)
+	{
+		QString text = QString("%1x%2").arg(QString::number(this->supportedRes[i].width), 
+			QString::number (this->supportedRes[i].height));
+		if (cbFullscreenResolution->findText(text) != -1)
+			continue;
+		cbFullscreenResolution->addItem(text, i);
+	}
+}
+
+void UISettings::loadTranslations()
+{
+	cbTranslation->addItem(QString::fromUtf8(_("Use System Locale")), "");
+	cbTranslation->addItem("English", "#");
+	for (int i = 0; i < this->trans.count(); i++)
+		cbTranslation->addItem(trans[i].name.left(1).toUpper()+trans[i].name.mid(1), trans[i].file);
+
+}
+
+void UISettings::loadShortcuts()
+{	
+	QList<QAction *> actions = parent()->findChildren<QAction *>();
+	foreach ( QAction* action, actions )
+	{
+		if (action->text().isEmpty())
+			continue;
+
+		actionsList.append(action);
+	}
+
+	int row=0;
+	twShortcuts->setRowCount(actionsList.count());
+	foreach ( QAction* action, actionsList )
+	{
+		QTableWidgetItem *tblItem = new QTableWidgetItem(QString::fromUtf8(_(action->text().toUtf8())));
+		tblItem->setFlags(tblItem->flags() ^ Qt::ItemIsEditable);
+		twShortcuts->setItem(row, 0, tblItem);
+		tblItem = new QTableWidgetItem(action->shortcut().toString());
+		twShortcuts->setItem(row, 1, tblItem);
+		row++;
+	}
+	QHeaderView *headerView = twShortcuts->horizontalHeader();
+	headerView->setResizeMode(QHeaderView::Stretch);
+	headerView->setResizeMode(1, QHeaderView::Interactive);
+}
+
+void UISettings::applyShortcuts()
+{
+	for (int row = 0; row < (int)actionsList.size(); ++row) 
+	{
+		QAction *action = actionsList[row];
+		action->setShortcut(QKeySequence(twShortcuts->item(row, 1)->text()));
+	}
+}
+
 void UISettings::loadSettings()
 {
 	// get settings pointer
@@ -222,21 +379,44 @@ void UISettings::loadSettings()
 	leBios->setText( s->value( "General/Bios" ).toString() );
 	cbCdRom->setCurrentIndex( cbCdRom->findData( s->value( "General/CdRom", QtYabause::defaultCDCore().id ).toInt() ) );
 	leCdRom->setText( s->value( "General/CdRomISO" ).toString() );
+	if (s->value( "General/CdRom", QtYabause::defaultCDCore().id ).toInt() == CDCORE_ARCH)
+		cbCdDrive->setCurrentIndex(leCdRom->text().isEmpty() ? 0 : cbCdDrive->findText(leCdRom->text()));
+
 	leSaveStates->setText( s->value( "General/SaveStates", getDataDirPath() ).toString() );
-	leTranslation->setText( s->value( "General/Translation" ).toString() );
+#ifdef HAVE_LIBMINI18N
+	int i;
+	if ((i=cbTranslation->findData(s->value( "General/Translation" ).toString())) != -1)
+		cbTranslation->setCurrentIndex(i);
+	else
+		cbTranslation->setCurrentIndex(0);
+#endif
 	cbEnableFrameSkipLimiter->setChecked( s->value( "General/EnableFrameSkipLimiter" ).toBool() );
 	cbShowFPS->setChecked( s->value( "General/ShowFPS" ).toBool() );
 	cbAutostart->setChecked( s->value( "autostart" ).toBool() );
 
+	bool clocksync = s->value( "General/ClockSync" ).toBool();
+	cbClockSync->setChecked( clocksync );	 
+	dteBaseTime->setVisible( clocksync );
+
+	QString dt = s->value( "General/FixedBaseTime" ).toString();
+	if (!dt.isEmpty())		
+		dteBaseTime->setDateTime( QDateTime::fromString( dt,Qt::ISODate) );
+	else
+		dteBaseTime->setDateTime( QDateTime(QDate(1998, 1, 1), QTime(12, 0, 0)) );
+
 	// video
 	cbVideoCore->setCurrentIndex( cbVideoCore->findData( s->value( "Video/VideoCore", QtYabause::defaultVIDCore().id ).toInt() ) );
 #if YAB_PORT_OSD
 	cbOSDCore->setCurrentIndex( cbOSDCore->findData( s->value( "Video/OSDCore", QtYabause::defaultOSDCore().id ).toInt() ) );
 #endif
-	leWidth->setText( s->value( "Video/Width" ).toString() );
-	leHeight->setText( s->value( "Video/Height" ).toString() );
+
+	leWinWidth->setText( s->value( "Video/WindowWidth", s->value( "Video/Width", 640 ) ).toString() );
+	leWinHeight->setText( s->value( "Video/WindowHeight", s->value( "Video/Height", 480 ) ).toString() );
+	QString text = QString("%1x%2").arg(s->value( "Video/FullscreenWidth", s->value( "Video/Width", 640 ) ).toString(),
+										s->value( "Video/FullscreenHeight", s->value( "Video/Height", 480 ) ).toString());	
+	cbFullscreenResolution->setCurrentIndex(cbFullscreenResolution->findText(text));
 	cbFullscreen->setChecked( s->value( "Video/Fullscreen", false ).toBool() );
-	cbVideoFormat->setCurrentIndex( cbVideoFormat->findData( s->value( "Video/VideoFormat", mVideoFromats.at( 0 ).id ).toInt() ) );
+	cbVideoFormat->setCurrentIndex( cbVideoFormat->findData( s->value( "Video/VideoFormat", mVideoFormats.at( 0 ).id ).toInt() ) );
 
 	// sound
 	cbSoundCore->setCurrentIndex( cbSoundCore->findData( s->value( "Sound/SoundCore", QtYabause::defaultSNDCore().id ).toInt() ) );
@@ -255,15 +435,20 @@ void UISettings::loadSettings()
 	cbSH2Interpreter->setCurrentIndex( cbSH2Interpreter->findData( s->value( "Advanced/SH2Interpreter", QtYabause::defaultSH2Core().id ).toInt() ) );
 
 	// view
-	bgShowMenubar->setId( rbMenubarNever, 0 );
-	bgShowMenubar->setId( rbMenubarFullscreen, 1 );
-	bgShowMenubar->setId( rbMenubarAlways, 2 );
-	bgShowMenubar->button( s->value( "View/Menubar", 0 ).toInt() )->setChecked( true );
+	bgShowMenubar->setId( rbMenubarNever, BD_NEVERHIDE );
+	bgShowMenubar->setId( rbMenubarFullscreen, BD_HIDEFS );
+	bgShowMenubar->setId( rbMenubarAlways, BD_ALWAYSHIDE );
+	bgShowMenubar->setId( rbMenubarFullscreenHover, BD_SHOWONFSHOVER );
+	bgShowMenubar->button( s->value( "View/Menubar", BD_SHOWONFSHOVER ).toInt() )->setChecked( true );
 
-	bgShowToolbar->setId( rbToolbarNever, 0 );
-	bgShowToolbar->setId( rbToolbarFullscreen, 1 );
-	bgShowToolbar->setId( rbToolbarAlways, 2 );
-	bgShowToolbar->button( s->value( "View/Toolbar", 1 ).toInt() )->setChecked( true );
+	bgShowToolbar->setId( rbToolbarNever, BD_NEVERHIDE );
+	bgShowToolbar->setId( rbToolbarFullscreen, BD_HIDEFS );
+	bgShowToolbar->setId( rbToolbarAlways, BD_ALWAYSHIDE );
+	bgShowToolbar->button( s->value( "View/Toolbar", BD_HIDEFS ).toInt() )->setChecked( true );
+
+	bgShowLogWindow->setId( rbLogWindowNever, 0 );
+	bgShowLogWindow->setId( rbLogWindowMessage, 1 );
+	bgShowLogWindow->button( s->value( "View/LogWindow", 0 ).toInt() )->setChecked( true );
 }
 
 void UISettings::saveSettings()
@@ -274,9 +459,15 @@ void UISettings::saveSettings()
 	// general
 	s->setValue( "General/Bios", leBios->text() );
 	s->setValue( "General/CdRom", cbCdRom->itemData( cbCdRom->currentIndex() ).toInt() );
-	s->setValue( "General/CdRomISO", leCdRom->text() );
+	CDInterface* core = QtYabause::getCDCore( cbCdRom->itemData( cbCdRom->currentIndex() ).toInt() );
+	if ( core->id == CDCORE_ARCH )
+		s->setValue( "General/CdRomISO", cbCdDrive->currentText() );
+	else
+		s->setValue( "General/CdRomISO", leCdRom->text() );
 	s->setValue( "General/SaveStates", leSaveStates->text() );
-	s->setValue( "General/Translation", leTranslation->text() );
+#ifdef HAVE_LIBMINI18N
+	s->setValue( "General/Translation", cbTranslation->itemData(cbTranslation->currentIndex()).toString() );
+#endif
 	s->setValue( "General/EnableFrameSkipLimiter", cbEnableFrameSkipLimiter->isChecked() );
 	s->setValue( "General/ShowFPS", cbShowFPS->isChecked() );
 	s->setValue( "autostart", cbAutostart->isChecked() );
@@ -286,11 +477,27 @@ void UISettings::saveSettings()
 #if YAB_PORT_OSD
 	s->setValue( "Video/OSDCore", cbOSDCore->itemData( cbOSDCore->currentIndex() ).toInt() );
 #endif
-	s->setValue( "Video/Width", leWidth->text() );
-	s->setValue( "Video/Height", leHeight->text() );
+	// Move Outdated window/fullscreen keys
+	s->remove("Video/Width");
+	s->remove("Video/Height");
+
+	// Save new version of keys
+	s->setValue( "Video/WindowWidth", leWinWidth->text() );
+	s->setValue( "Video/WindowHeight", leWinHeight->text() );
+
+	if (supportedRes.count() > 0)
+	{
+		supportedRes_struct res = supportedRes[cbFullscreenResolution->itemData(cbFullscreenResolution->currentIndex()).toInt()];
+		s->setValue( "Video/FullscreenWidth", res.width );
+		s->setValue( "Video/FullscreenHeight", res.height );
+	}
+
 	s->setValue( "Video/Fullscreen", cbFullscreen->isChecked() );
 	s->setValue( "Video/VideoFormat", cbVideoFormat->itemData( cbVideoFormat->currentIndex() ).toInt() );
 
+	s->setValue( "General/ClockSync", cbClockSync->isChecked() );
+	s->setValue( "General/FixedBaseTime", dteBaseTime->dateTime().toString(Qt::ISODate));
+
 	// sound
 	s->setValue( "Sound/SoundCore", cbSoundCore->itemData( cbSoundCore->currentIndex() ).toInt() );
 
@@ -310,6 +517,21 @@ void UISettings::saveSettings()
 	// view
 	s->setValue( "View/Menubar", bgShowMenubar->checkedId() );
 	s->setValue( "View/Toolbar", bgShowToolbar->checkedId() );
+	s->setValue( "View/LogWindow", bgShowLogWindow->checkedId() );
+
+	// shortcuts
+	applyShortcuts();
+	s->beginGroup("Shortcuts");
+	QList<QAction *> actions = parent()->findChildren<QAction *>();
+	foreach ( QAction* action, actions )
+	{
+		if (action->text().isEmpty())
+			continue;
+
+		QString accelText = QString(action->shortcut());
+		s->setValue(action->text(), accelText);
+	}
+	s->endGroup();
 }
 
 void UISettings::accept()
