@@ -1,5 +1,5 @@
 /*  Copyright 2005 Guillaume Duhamel
-	Copyright 2005-2006 Theo Berkau
+	Copyright 2005-2006, 2013 Theo Berkau
 	Copyright 2008 Filipe Azevedo <pasnox@gmail.com>
 
 	This file is part of Yabause.
@@ -31,6 +31,7 @@
 #include "UIDebugM68K.h"
 #include "UIDebugSCUDSP.h"
 #include "UIDebugSCSP.h"
+#include "UIMemoryEditor.h"
 #include "UIMemoryTransfer.h"
 #include "UIAbout.h"
 #include "../YabauseGL.h"
@@ -69,8 +70,7 @@ UIYabause::UIYabause( QWidget* parent )
 	: QMainWindow( parent )
 {
 	mInit = false;
-	searchResults = NULL;
-	numSearchResults = 0;
+   search.clear();
 	searchType = 0;
 
 	// setup dialog
@@ -86,6 +86,8 @@ UIYabause::UIYabause( QWidget* parent )
 	fVideoDriver->setParent( 0, Qt::Popup );
 	fSound->installEventFilter( this );
 	fVideoDriver->installEventFilter( this );
+	// Get Screen res list
+	getSupportedResolutions();
 	// fill combo driver
 	cbVideoDriver->blockSignals( true );
 	for ( int i = 0; VIDCoreList[i] != NULL; i++ )
@@ -111,11 +113,27 @@ UIYabause::UIYabause( QWidget* parent )
 	// connectionsdd
 	connect( mYabauseThread, SIGNAL( requestSize( const QSize& ) ), this, SLOT( sizeRequested( const QSize& ) ) );
 	connect( mYabauseThread, SIGNAL( requestFullscreen( bool ) ), this, SLOT( fullscreenRequested( bool ) ) );
+	connect( mYabauseThread, SIGNAL( requestVolumeChange( int ) ), this, SLOT( on_sVolume_valueChanged( int ) ) );
 	connect( aViewLog, SIGNAL( toggled( bool ) ), mLogDock, SLOT( setVisible( bool ) ) );
 	connect( mLogDock->toggleViewAction(), SIGNAL( toggled( bool ) ), aViewLog, SLOT( setChecked( bool ) ) );
 	connect( mYabauseThread, SIGNAL( error( const QString&, bool ) ), this, SLOT( errorReceived( const QString&, bool ) ) );
 	connect( mYabauseThread, SIGNAL( pause( bool ) ), this, SLOT( pause( bool ) ) );
 	connect( mYabauseThread, SIGNAL( reset() ), this, SLOT( reset() ) );
+
+	// Load shortcuts
+	VolatileSettings* vs = QtYabause::volatileSettings();
+	QList<QAction *> actions = findChildren<QAction *>();
+	foreach ( QAction* action, actions )
+	{
+		if (action->text().isEmpty())
+			continue;
+
+		QString text = vs->value(QString("Shortcuts/") + action->text(), "").toString();
+		if (text.isEmpty())
+			continue;
+		action->setShortcut(text);
+	}
+
 	// retranslate widgets
 	QtYabause::retranslateWidget( this );
 
@@ -123,6 +141,12 @@ UIYabause::UIYabause( QWidget* parent )
 	for(int i = 0;i < actionList.size();i++) {
 		addAction(actionList.at(i));
 	}
+
+	restoreGeometry( vs->value("General/Geometry" ).toByteArray() );
+	mYabauseGL->setMouseTracking(true);
+	setMouseTracking(true);
+	showMenuBarHeight = menubar->height();
+	translations = QtYabause::getTranslationList();
 }
 
 void UIYabause::showEvent( QShowEvent* e )
@@ -134,9 +158,10 @@ void UIYabause::showEvent( QShowEvent* e )
 		LogStart();
 		LogChangeOutput( DEBUG_CALLBACK, (char*)qAppendLog );
 		VolatileSettings* vs = QtYabause::volatileSettings();
-		if ( vs->value( "View/Menubar" ).toInt() == 2 )
+
+		if ( vs->value( "View/Menubar" ).toInt() == BD_ALWAYSHIDE )
 			menubar->hide();
-		if ( vs->value( "View/Toolbar" ).toInt() == 2 )
+		if ( vs->value( "View/Toolbar" ).toInt() == BD_ALWAYSHIDE )
 			toolBar->hide();
 		if ( vs->value( "autostart" ).toBool() )
 			aEmulationRun->trigger();
@@ -150,6 +175,14 @@ void UIYabause::closeEvent( QCloseEvent* e )
 {
 	aEmulationPause->trigger();
 	LogStop();
+
+	if (isFullScreen())
+		// Need to switch out of full screen or the geometry settings get saved
+		fullscreenRequested( false );
+	Settings* vs = QtYabause::settings();
+	vs->setValue( "General/Geometry", saveGeometry() );
+	vs->sync();
+
 	QMainWindow::closeEvent( e );
 }
 
@@ -159,15 +192,44 @@ void UIYabause::keyPressEvent( QKeyEvent* e )
 void UIYabause::keyReleaseEvent( QKeyEvent* e )
 { PerKeyUp( e->key() ); }
 
+void UIYabause::mousePressEvent( QMouseEvent* e )
+{ 
+	PerKeyDown( (1 << 31) | e->button() );
+}
+
+void UIYabause::mouseReleaseEvent( QMouseEvent* e )
+{ 
+	PerKeyUp( (1 << 31) | e->button() );
+}
+
+void UIYabause::mouseMoveEvent( QMouseEvent* e )
+{ 
+	PerAxisMove((1 << 30), e->x()-oldMouseX, oldMouseY-e->y());
+	oldMouseX = e->x();
+	oldMouseY = e->y();
+	VolatileSettings* vs = QtYabause::volatileSettings();
+	if (isFullScreen() && vs->value( "View/Menubar" ).toInt() == BD_SHOWONFSHOVER )
+	{
+		if (e->y() < showMenuBarHeight)
+			menubar->show();
+		else
+			menubar->hide();
+	}
+}
+
 void UIYabause::swapBuffers()
-{ mYabauseGL->swapBuffers(); }
+{ 
+	mYabauseGL->swapBuffers(); 
+	mYabauseGL->makeCurrent();
+}
 
 void UIYabause::appendLog( const char* s )
 {
 	teLog->moveCursor( QTextCursor::End );
 	teLog->append( s );
-	
-	if ( !mLogDock->isVisible() ) {
+
+	VolatileSettings* vs = QtYabause::volatileSettings();
+	if (( !mLogDock->isVisible( )) && ( vs->value( "View/LogWindow" ).toInt() == 1 )) {
 		mLogDock->setVisible( true );
 	}
 }
@@ -190,20 +252,150 @@ void UIYabause::errorReceived( const QString& error, bool internal )
 }
 
 void UIYabause::sizeRequested( const QSize& s )
-{ resize( s.isNull() ? QSize( 640, 480 ) : s ); }
+{
+	int heightOffset = toolBar->height()+menubar->height();
+	int width, height;
+	if (s.isNull())
+	{
+		width=640;
+		height=480;
+	}
+	else
+	{
+		width=s.width();
+		height=s.height();
+	}
+	// Compensate for menubar and toolbar
+	VolatileSettings* vs = QtYabause::volatileSettings();
+	if (vs->value( "View/Menubar" ).toInt() != BD_ALWAYSHIDE)
+		height += menubar->height();
+	if (vs->value( "View/Toolbar" ).toInt() != BD_ALWAYSHIDE)
+		height += toolBar->height();
+	resize( width, height ); 
+}
+
+void UIYabause::getSupportedResolutions()
+{
+#if defined Q_OS_WIN
+	DEVMODE devMode;
+	BOOL result = TRUE;
+	DWORD currentSettings = 0;
+	devMode.dmSize = sizeof(DEVMODE);
+
+	supportedResolutions.clear();
+
+	while (result)
+	{
+		result = EnumDisplaySettings(NULL, currentSettings, &devMode);
+		if (result && devMode.dmBitsPerPel == 32)
+		{
+			supportedRes_struct res;
+			res.width = devMode.dmPelsWidth;
+			res.height = devMode.dmPelsHeight;
+			res.bpp = devMode.dmBitsPerPel;
+			res.freq = devMode.dmDisplayFrequency;
+
+			supportedResolutions.append(res);
+		}
+		currentSettings++;
+	}
+#elif HAVE_LIBXRANDR
+	ResolutionList list;
+	supportedRes_struct res;
+
+	list = ScreenGetResolutions();
+
+	while(0 == ScreenNextResolution(list, &res))
+		supportedResolutions.append(res);
+#endif
+}
+
+int UIYabause::isResolutionValid( int width, int height, int bpp, int freq )
+{
+	for (int i = 0; i < supportedResolutions.count(); i++)
+	{
+		if (supportedResolutions[i].width == width &&
+			supportedResolutions[i].height == height)
+			return i;
+	}
+
+	return -1;
+}
+
+int UIYabause::findBestVideoFreq( int width, int height, int bpp, int videoFormat )
+{
+	// Try to use a frequency close to 60 hz for NTSC, 75 hz for PAL
+	if (videoFormat == VIDEOFORMATTYPE_PAL && isResolutionValid( width, height, bpp, 75 ) > 0)
+		return 75;
+	else if (videoFormat == VIDEOFORMATTYPE_NTSC && isResolutionValid( width, height, bpp, 60 ) > 0)
+		return 60;
+	else
+	{
+		// Since we can't use the frequency we want, use the first one available
+		int i=isResolutionValid( width, height, bpp, -1 );
+		if (i < 0)
+			return -1;
+		return supportedResolutions[i].freq;
+	}
+}
+
+void UIYabause::toggleFullscreen( int width, int height, bool f, int videoFormat )
+{
+	// Make sure setting is valid
+	if (f && isResolutionValid( width, height, -1, -1 ) < 0)
+		return;
+
+#if defined Q_OS_WIN
+	if (f)
+	{
+		DEVMODE dmScreenSettings;
+		memset (&dmScreenSettings, 0, sizeof (dmScreenSettings));
+
+		int freq = findBestVideoFreq( width, height, 32, videoFormat );
+
+		if (freq < 0)
+			return;
+
+		dmScreenSettings.dmSize = sizeof (dmScreenSettings);   
+		dmScreenSettings.dmPelsWidth = width;
+		dmScreenSettings.dmPelsHeight = height;    
+		dmScreenSettings.dmBitsPerPel = 32;
+		dmScreenSettings.dmDisplayFrequency = freq;
+		dmScreenSettings.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
+		ChangeDisplaySettings(&dmScreenSettings, CDS_FULLSCREEN);
+	}
+	else
+		ChangeDisplaySettings(NULL, 0);
+
+#elif HAVE_LIBXRANDR
+	if (f)
+	{
+		int i = isResolutionValid(width, height, 32, -1);
+		ScreenChangeResolution(&supportedResolutions[i]);
+	}
+	else
+	{
+		ScreenRestoreResolution();
+	}
+#endif
+}
 
 void UIYabause::fullscreenRequested( bool f )
 {
 	if ( isFullScreen() && !f )
 	{
-		showNormal();
 #ifdef USE_UNIFIED_TITLE_TOOLBAR
 		setUnifiedTitleAndToolBarOnMac( true );
 #endif
+		toggleFullscreen(0, 0, false, -1 );
+		showNormal();
+
 		VolatileSettings* vs = QtYabause::volatileSettings();
-		if ( vs->value( "View/Menubar" ).toInt() == 1 )
+		int menubarHide = vs->value( "View/Menubar" ).toInt();
+		if ( menubarHide == BD_HIDEFS ||
+			  menubarHide == BD_SHOWONFSHOVER)
 			menubar->show();
-		if ( vs->value( "View/Toolbar" ).toInt() == 1 )
+		if ( vs->value( "View/Toolbar" ).toInt() == BD_HIDEFS )
 			toolBar->show();
 	}
 	else if ( !isFullScreen() && f )
@@ -211,11 +403,15 @@ void UIYabause::fullscreenRequested( bool f )
 #ifdef USE_UNIFIED_TITLE_TOOLBAR
 		setUnifiedTitleAndToolBarOnMac( false );
 #endif
-		showFullScreen();
 		VolatileSettings* vs = QtYabause::volatileSettings();
-		if ( vs->value( "View/Menubar" ).toInt() == 1 )
+
+		toggleFullscreen(vs->value("Video/FullscreenWidth").toInt(), vs->value("Video/FullscreenHeight").toInt(), 
+						f, vs->value("Video/VideoFormat").toInt());
+		showFullScreen();
+
+		if ( vs->value( "View/Menubar" ).toInt() == BD_HIDEFS )
 			menubar->hide();
-		if ( vs->value( "View/Toolbar" ).toInt() == 1 )
+		if ( vs->value( "View/Toolbar" ).toInt() == BD_HIDEFS )
 			toolBar->hide();
 	}
 	if ( aViewFullscreen->isChecked() != f )
@@ -279,32 +475,114 @@ void UIYabause::refreshStatesActions()
 
 void UIYabause::on_aFileSettings_triggered()
 {
+	Settings *s = (QtYabause::settings());
+	QHash<QString, QVariant> hash;
+	const QStringList keys = s->allKeys();
+	Q_FOREACH(QString key, keys) {
+		hash[key] = s->value(key);
+	}
+
 	YabauseLocker locker( mYabauseThread );
-	if ( UISettings( window() ).exec() )
+	if ( UISettings( &supportedResolutions, &translations, window() ).exec() )
 	{
 		VolatileSettings* vs = QtYabause::volatileSettings();
 		aEmulationFrameSkipLimiter->setChecked( vs->value( "General/EnableFrameSkipLimiter" ).toBool() );
 		aViewFPS->setChecked( vs->value( "General/ShowFPS" ).toBool() );
-
-		if ( vs->value( "View/Menubar" ).toInt() == 2 )
-			menubar->hide();
-		else
-			menubar->show();
-
-		if ( vs->value( "View/Toolbar" ).toInt() == 2 )
-			toolBar->hide();
-		else
-			toolBar->show();
 		
-		if ( mYabauseThread->pauseEmulation( true, true ) )
-			refreshStatesActions();
+		if(isFullScreen())
+		{
+			if ( vs->value( "View/Menubar" ).toInt() == BD_HIDEFS || vs->value( "View/Menubar" ).toInt() == BD_ALWAYSHIDE )
+				menubar->hide();
+			else
+				menubar->show();
+
+			if ( vs->value( "View/Toolbar" ).toInt() == BD_HIDEFS || vs->value( "View/Toolbar" ).toInt() == BD_ALWAYSHIDE )
+				toolBar->hide();
+			else
+				toolBar->show();
+		}
+		else
+		{
+			if ( vs->value( "View/Menubar" ).toInt() == BD_ALWAYSHIDE )
+				menubar->hide();
+			else
+				menubar->show();
+
+			if ( vs->value( "View/Toolbar" ).toInt() == BD_ALWAYSHIDE )
+				toolBar->hide();
+			else
+				toolBar->show();
+		}
+
+		
+		//only reset if bios, region, cart,  back up, mpeg, sh2, m68k are changed
+		Settings *ss = (QtYabause::settings());
+		QHash<QString, QVariant> newhash;
+		const QStringList newkeys = ss->allKeys();
+		Q_FOREACH(QString key, newkeys) {
+			newhash[key] = ss->value(key);
+		}
+		if(newhash["General/Bios"]!=hash["General/Bios"] ||
+			newhash["Advanced/Region"]!=hash["Advanced/Region"] ||
+			newhash["Cartridge/Type"]!=hash["Cartridge/Type"] ||
+			newhash["Memory/Path"]!=hash["Memory/Path"] ||
+			newhash["MpegROM/Path" ]!=hash["MpegROM/Path" ] ||
+			newhash["Advanced/SH2Interpreter" ]!=hash["Advanced/SH2Interpreter" ] ||
+			newhash["General/CdRom"]!=hash["General/CdRom"] ||
+			newhash["General/CdRomISO"]!=hash["General/CdRomISO"] ||
+			newhash["General/ClockSync"]!=hash["General/ClockSync"] ||
+			newhash["General/FixedBaseTime"]!=hash["General/FixedBaseTime"]
+		)
+		{
+			if ( mYabauseThread->pauseEmulation( true, true ) )
+				refreshStatesActions();
+			return;
+		}
+#ifdef HAVE_LIBMINI18N
+		if(newhash["General/Translation"] != hash["General/Translation"])
+		{
+			mini18n_close();
+			retranslateUi(this);
+			if ( QtYabause::setTranslationFile() == -1 )
+				qWarning( "Can't set translation file" );
+			QtYabause::retranslateApplication();
+		}
+#endif
+		if(newhash["Video/VideoCore"] != hash["Video/VideoCore"])
+			on_cbVideoDriver_currentIndexChanged(newhash["Video/VideoCore"].toInt());
+		
+		if(newhash["General/ShowFPS"] != hash["General/ShowFPS"])
+			SetOSDToggle(newhash["General/ShowFPS"].toBool());
+		
+		if (newhash["Sound/SoundCore"] != hash["Sound/SoundCore"])
+			ScspChangeSoundCore(newhash["Sound/SoundCore"].toInt());
+		
+		if (newhash["Video/WindowWidth"] != hash["Video/WindowWidth"] || newhash["Video/WindowHeight"] != hash["Video/WindowHeight"] ||
+          newhash["View/Menubar"] != hash["View/Menubar"] || newhash["View/Toolbar"] != hash["View/Toolbar"])
+			sizeRequested(QSize(newhash["Video/WindowWidth"].toInt(),newhash["Video/WindowHeight"].toInt()));
+		
+		if (newhash["Video/FullscreenWidth"] != hash["Video/FullscreenWidth"] || 
+			newhash["Video/FullscreenHeight"] != hash["Video/FullscreenHeight"] ||
+			newhash["Video/Fullscreen"] != hash["Video/Fullscreen"])
+		{
+			bool f = isFullScreen();
+			if (f)
+				fullscreenRequested( false );
+			fullscreenRequested( f );
+		}
+		
+		if (newhash["Video/VideoFormat"] != hash["Video/VideoFormat"])
+			YabauseSetVideoFormat(newhash["Video/VideoFormat"].toInt());
+
+		mYabauseThread->reloadControllers();
+		refreshStatesActions();
 	}
 }
 
 void UIYabause::on_aFileOpenISO_triggered()
 {
 	YabauseLocker locker( mYabauseThread );
-	const QString fn = CommonDialogs::getOpenFileName( QtYabause::volatileSettings()->value( "Recents/ISOs" ).toString(), QtYabause::translate( "Select your iso/cue/bin file" ), QtYabause::translate( "CD Images (*.iso *.cue *.bin)" ) );
+	const QString fn = CommonDialogs::getOpenFileName( QtYabause::volatileSettings()->value( "Recents/ISOs" ).toString(), QtYabause::translate( "Select your iso/cue/bin file" ), QtYabause::translate( "CD Images (*.iso *.cue *.bin *.mds)" ) );
 	if ( !fn.isEmpty() )
 	{
 		VolatileSettings* vs = QtYabause::volatileSettings();
@@ -326,21 +604,25 @@ void UIYabause::on_aFileOpenISO_triggered()
 void UIYabause::on_aFileOpenCDRom_triggered()
 {
 	YabauseLocker locker( mYabauseThread );
-	const QString fn = CommonDialogs::getExistingDirectory( QtYabause::volatileSettings()->value( "Recents/CDs" ).toString(), QtYabause::translate( "Choose a cdrom drive/mount point" ) );
-	if ( !fn.isEmpty() )
+	QStringList list = getCdDriveList();
+	int current = list.indexOf(QtYabause::volatileSettings()->value( "Recents/CDs").toString());
+	QString fn = QInputDialog::getItem(this, QtYabause::translate("Open CD Rom"), 
+													QtYabause::translate("Choose a cdrom drive/mount point") + ":",
+													list, current, false);
+	if (!fn.isEmpty())
 	{
 		VolatileSettings* vs = QtYabause::volatileSettings();
 		const int currentCDCore = vs->value( "General/CdRom" ).toInt();
 		const QString currentCdRomISO = vs->value( "General/CdRomISO" ).toString();
-		
+
 		QtYabause::settings()->setValue( "Recents/CDs", fn );
-		
+
 		vs->setValue( "autostart", false );
 		vs->setValue( "General/CdRom", QtYabause::defaultCDCore().id );
 		vs->setValue( "General/CdRomISO", fn );
-		
+
 		mYabauseThread->pauseEmulation( false, true );
-		
+
 		refreshStatesActions();
 	}
 }
@@ -350,7 +632,7 @@ void UIYabause::on_mFileSaveState_triggered( QAction* a )
 	if ( a == aFileSaveStateAs )
 		return;
 	YabauseLocker locker( mYabauseThread );
-	if ( YabSaveStateSlot( QtYabause::volatileSettings()->value( "General/SaveStates", getDataDirPath() ).toString().toAscii().constData(), a->data().toInt() ) != 0 )
+	if ( YabSaveStateSlot( QtYabause::volatileSettings()->value( "General/SaveStates", getDataDirPath() ).toString().toLatin1().constData(), a->data().toInt() ) != 0 )
 		CommonDialogs::information( QtYabause::translate( "Couldn't save state file" ) );
 	else
 		refreshStatesActions();
@@ -361,7 +643,7 @@ void UIYabause::on_mFileLoadState_triggered( QAction* a )
 	if ( a == aFileLoadStateAs )
 		return;
 	YabauseLocker locker( mYabauseThread );
-	if ( YabLoadStateSlot( QtYabause::volatileSettings()->value( "General/SaveStates", getDataDirPath() ).toString().toAscii().constData(), a->data().toInt() ) != 0 )
+	if ( YabLoadStateSlot( QtYabause::volatileSettings()->value( "General/SaveStates", getDataDirPath() ).toString().toLatin1().constData(), a->data().toInt() ) != 0 )
 		CommonDialogs::information( QtYabause::translate( "Couldn't load state file" ) );
 }
 
@@ -371,7 +653,7 @@ void UIYabause::on_aFileSaveStateAs_triggered()
 	const QString fn = CommonDialogs::getSaveFileName( QtYabause::volatileSettings()->value( "General/SaveStates", getDataDirPath() ).toString(), QtYabause::translate( "Choose a file to save your state" ), QtYabause::translate( "Yabause Save State (*.yss)" ) );
 	if ( fn.isNull() )
 		return;
-	if ( YabSaveState( fn.toAscii().constData() ) != 0 )
+	if ( YabSaveState( fn.toLatin1().constData() ) != 0 )
 		CommonDialogs::information( QtYabause::translate( "Couldn't save state file" ) );
 }
 
@@ -381,7 +663,7 @@ void UIYabause::on_aFileLoadStateAs_triggered()
 	const QString fn = CommonDialogs::getOpenFileName( QtYabause::volatileSettings()->value( "General/SaveStates", getDataDirPath() ).toString(), QtYabause::translate( "Select a file to load your state" ), QtYabause::translate( "Yabause Save State (*.yss)" ) );
 	if ( fn.isNull() )
 		return;
-	if ( YabLoadState( fn.toAscii().constData() ) != 0 )
+	if ( YabLoadState( fn.toLatin1().constData() ) != 0 )
 		CommonDialogs::information( QtYabause::translate( "Couldn't load state file" ) );
 	else
 		aEmulationRun->trigger();
@@ -397,7 +679,11 @@ void UIYabause::on_aFileScreenshot_triggered()
 			filters << QString( ba ).toLower();
 	for ( int i = 0; i < filters.count(); i++ )
 		filters[i] = QtYabause::translate( "%1 Images (*.%2)" ).arg( filters[i].toUpper() ).arg( filters[i] );
-	
+
+#if defined(HAVE_LIBGL) && !defined(QT_OPENGL_ES_1) && !defined(QT_OPENGL_ES_2)
+	glReadBuffer(GL_FRONT);
+#endif
+
 	// take screenshot of gl view
 	QImage screenshot = mYabauseGL->grabFrameBuffer();
 	
@@ -416,7 +702,10 @@ void UIYabause::on_aFileQuit_triggered()
 void UIYabause::on_aEmulationRun_triggered()
 {
 	if ( mYabauseThread->emulationPaused() )
+   {
 		mYabauseThread->pauseEmulation( false, false );
+      refreshStatesActions();
+   }
 }
 
 void UIYabause::on_aEmulationPause_triggered()
@@ -430,6 +719,10 @@ void UIYabause::on_aEmulationReset_triggered()
 
 void UIYabause::on_aEmulationFrameSkipLimiter_toggled( bool toggled )
 {
+	Settings* vs = QtYabause::settings();
+	vs->setValue( "General/EnableFrameSkipLimiter", toggled );
+	vs->sync();
+
 	if ( toggled )
 		EnableAutoFrameSkip();
 	else
@@ -456,11 +749,11 @@ void UIYabause::on_aToolsCheatsList_triggered()
 void UIYabause::on_aToolsCheatSearch_triggered()
 {
    YabauseLocker locker( mYabauseThread );
-   UICheatSearch cs(this, searchResults, numSearchResults, searchType);
+   UICheatSearch cs(this, &search, searchType);
       
    cs.exec();
 
-   searchResults = cs.getSearchVariables(&numSearchResults, &searchType);
+   search = *cs.getSearchVariables( &searchType);
 }
 
 void UIYabause::on_aToolsTransfer_triggered()
@@ -471,6 +764,9 @@ void UIYabause::on_aToolsTransfer_triggered()
 
 void UIYabause::on_aViewFPS_triggered( bool toggled )
 {
+	Settings* vs = QtYabause::settings();
+	vs->setValue( "General/ShowFPS", toggled );
+	vs->sync();
 	SetOSDToggle(toggled ? 1 : 0);
 }
 
@@ -498,17 +794,19 @@ void UIYabause::on_aViewFullscreen_triggered( bool b )
 	//ToggleFullScreen();
 }
 
-void UIYabause::breakpointHandlerMSH2()
+void UIYabause::breakpointHandlerMSH2(bool displayMessage)
 {
 	YabauseLocker locker( mYabauseThread );
-	CommonDialogs::information( QtYabause::translate( "Breakpoint Reached" ) );
+	if (displayMessage)
+		CommonDialogs::information( QtYabause::translate( "Breakpoint Reached" ) );
 	UIDebugSH2( true, mYabauseThread, this ).exec();
 }
 
-void UIYabause::breakpointHandlerSSH2()
+void UIYabause::breakpointHandlerSSH2(bool displayMessage)
 {
 	YabauseLocker locker( mYabauseThread );
-	CommonDialogs::information( QtYabause::translate( "Breakpoint Reached" ) );
+	if (displayMessage)
+		CommonDialogs::information( QtYabause::translate( "Breakpoint Reached" ) );
 	UIDebugSH2( false, mYabauseThread, this ).exec();
 }
 
@@ -571,7 +869,7 @@ void UIYabause::on_aViewDebugSCSP_triggered()
 void UIYabause::on_aViewDebugMemoryEditor_triggered()
 {
 	YabauseLocker locker( mYabauseThread );
-	//UIMemoryEditor( this ).exec();
+	UIMemoryEditor( mYabauseThread, this ).exec();
 }
 
 void UIYabause::on_aHelpEmuCompatibility_triggered()
@@ -623,7 +921,11 @@ void UIYabause::on_cbSound_toggled( bool toggled )
 }
 
 void UIYabause::on_sVolume_valueChanged( int value )
-{ ScspSetVolume( value ); }
+{ 
+	ScspSetVolume( value ); 
+	Settings* vs = QtYabause::settings();
+	vs->setValue("Sound/Volume", value );
+}
 
 void UIYabause::on_cbVideoDriver_currentIndexChanged( int id )
 {
