@@ -1,5 +1,5 @@
 /*  Copyright 2005 Guillaume Duhamel
-	Copyright 2005-2006 Theo Berkau
+	Copyright 2005-2006, 2013 Theo Berkau
 	Copyright 2008 Filipe Azevedo <pasnox@gmail.com>
 
 	This file is part of Yabause.
@@ -23,11 +23,24 @@
 
 #include "ui_UIYabause.h"
 #include "../YabauseThread.h"
+#include "UICheatSearch.h"
+
+extern "C" {
+#include "../screen.h"
+}
 
 class YabauseGL;
 class QTextEdit;
 class QDockWidget;
 
+enum BARDISPLAY
+{
+	BD_NEVERHIDE=0,
+	BD_HIDEFS=1,
+	BD_ALWAYSHIDE=2,
+	BD_SHOWONFSHOVER=3
+};
+
 class YabauseLocker
 {
 public:
@@ -70,27 +83,36 @@ protected:
 	QDockWidget* mLogDock;
 	QTextEdit* teLog;
 	bool mInit;
-   result_struct *searchResults;
-   u32 numSearchResults;
-   int searchType;
-
+	QList <cheatsearch_struct> search;
+	int searchType;
+	QList <supportedRes_struct> supportedResolutions;
+	int oldMouseX, oldMouseY;
+	int showMenuBarHeight;
+	QList <translation_struct> translations;
 	virtual void showEvent( QShowEvent* event );
 	virtual void closeEvent( QCloseEvent* event );
 	virtual void keyPressEvent( QKeyEvent* event );
 	virtual void keyReleaseEvent( QKeyEvent* event );
+	virtual void mousePressEvent( QMouseEvent* event );
+	virtual void mouseReleaseEvent( QMouseEvent* event );
+	virtual void mouseMoveEvent( QMouseEvent* event );
 
 public slots:
 	void appendLog( const char* msg );
 	void pause( bool paused );
 	void reset();
 
-	void breakpointHandlerMSH2();
-	void breakpointHandlerSSH2();
+	void breakpointHandlerMSH2(bool displayMessage);
+	void breakpointHandlerSSH2(bool displayMessage);
 	void breakpointHandlerM68K();
 	void breakpointHandlerSCUDSP();
 protected slots:
 	void errorReceived( const QString& error, bool internal = true );
 	void sizeRequested( const QSize& size );
+	void getSupportedResolutions();
+	int isResolutionValid( int width, int height, int bpp, int freq );
+	int findBestVideoFreq( int width, int height, int bpp, int videoFormat );
+	void toggleFullscreen( int width, int height, bool f, int videoFormat );
 	void fullscreenRequested( bool fullscreen );
 	void refreshStatesActions();
 	// file menu
@@ -111,7 +133,7 @@ protected slots:
 	// tools
 	void on_aToolsBackupManager_triggered();
 	void on_aToolsCheatsList_triggered();
-   void on_aToolsCheatSearch_triggered();
+	void on_aToolsCheatSearch_triggered();
 	void on_aToolsTransfer_triggered();
 	// view menu
 	void on_aViewFPS_triggered( bool toggled );
