@@ -193,12 +193,6 @@ int VideoChangeCore(int coreid)
    // Reset resolution/priority variables
    if (Vdp2Regs)
    {
-      VIDCore->Vdp2SetResolution(Vdp2Regs->TVMD);
-      VIDCore->Vdp2SetPriorityNBG0(Vdp2Regs->PRINA & 0x7);
-      VIDCore->Vdp2SetPriorityNBG1((Vdp2Regs->PRINA >> 8) & 0x7);
-      VIDCore->Vdp2SetPriorityNBG2(Vdp2Regs->PRINB & 0x7);
-      VIDCore->Vdp2SetPriorityNBG3((Vdp2Regs->PRINB >> 8) & 0x7);
-      VIDCore->Vdp2SetPriorityRBG0(Vdp2Regs->PRIR & 0x7);
       VIDCore->Vdp1Reset();
    }
 
@@ -281,7 +275,9 @@ void FASTCALL Vdp1WriteWord(u32 addr, u16 val) {
             Vdp1External.manualerase = 1;
          break;
       case 0x4:
+         Vdp1Regs->COPR = 0;
          Vdp1Regs->PTMR = val;
+         if (val == 1) Vdp1Draw();
          break;
       case 0x6:
          Vdp1Regs->EWDR = val;
@@ -316,9 +312,6 @@ void Vdp1Draw(void) {
 
    VIDCore->Vdp1DrawStart();
 
-   if (!Vdp1Regs->PTMR)
-      return;
-
    if (!Vdp1External.disptoggle)
    {
       Vdp1NoDraw();
@@ -333,6 +326,8 @@ void Vdp1Draw(void) {
    // BEF <- CEF
    // CEF <- 0
    Vdp1Regs->EDSR >>= 1;
+   /* this should be done after a frame change or a plot trigger */
+   Vdp1Regs->COPR = 0;
 
    command = T1ReadWord(Vdp1Ram, Vdp1Regs->addr);
 
@@ -423,10 +418,6 @@ void Vdp1NoDraw(void) {
    u32 commandCounter;
    u16 command;
 
-   // Only when PTMR's not set do we not parse commands
-   if (!Vdp1Regs->PTMR)
-      return;
-
    Vdp1Regs->addr = 0;
    returnAddr = 0xFFFFFFFF;
    commandCounter = 0;
@@ -435,6 +426,8 @@ void Vdp1NoDraw(void) {
    // BEF <- CEF
    // CEF <- 0
    Vdp1Regs->EDSR >>= 1;
+   /* this should be done after a frame change or a plot trigger */
+   Vdp1Regs->COPR = 0;
 
    command = T1ReadWord(Vdp1Ram, Vdp1Regs->addr);
 
@@ -1321,13 +1314,7 @@ int VIDDummyVdp2Reset(void);
 void VIDDummyVdp2DrawStart(void);
 void VIDDummyVdp2DrawEnd(void);
 void VIDDummyVdp2DrawScreens(void);
-void VIDDummyVdp2SetResolution(u16 TVMD);
-void FASTCALL VIDDummyVdp2SetPriorityNBG0(int priority);
-void FASTCALL VIDDummyVdp2SetPriorityNBG1(int priority);
-void FASTCALL VIDDummyVdp2SetPriorityNBG2(int priority);
-void FASTCALL VIDDummyVdp2SetPriorityNBG3(int priority);
-void FASTCALL VIDDummyVdp2SetPriorityRBG0(int priority);
-void VIDDummyOnScreenDebugMessage(char * string, ...);
+void VIDDummyGetGlSize(int *width, int *height);
 
 
 VideoInterface_struct VIDDummy = {
@@ -1353,13 +1340,7 @@ VIDDummyVdp2Reset,
 VIDDummyVdp2DrawStart,
 VIDDummyVdp2DrawEnd,
 VIDDummyVdp2DrawScreens,
-VIDDummyVdp2SetResolution,
-VIDDummyVdp2SetPriorityNBG0,
-VIDDummyVdp2SetPriorityNBG1,
-VIDDummyVdp2SetPriorityNBG2,
-VIDDummyVdp2SetPriorityNBG3,
-VIDDummyVdp2SetPriorityRBG0,
-VIDDummyOnScreenDebugMessage
+VIDDummyGetGlSize
 };
 
 //////////////////////////////////////////////////////////////////////////////
@@ -1488,42 +1469,8 @@ void VIDDummyVdp2DrawScreens(void)
 
 //////////////////////////////////////////////////////////////////////////////
 
-void VIDDummyVdp2SetResolution(UNUSED u16 TVMD)
-{
-}
-
-//////////////////////////////////////////////////////////////////////////////
-
-void FASTCALL VIDDummyVdp2SetPriorityNBG0(UNUSED int priority)
-{
-}
-
-//////////////////////////////////////////////////////////////////////////////
-
-void FASTCALL VIDDummyVdp2SetPriorityNBG1(UNUSED int priority)
-{
-}
-
-//////////////////////////////////////////////////////////////////////////////
-
-void FASTCALL VIDDummyVdp2SetPriorityNBG2(UNUSED int priority)
-{
-}
-
-//////////////////////////////////////////////////////////////////////////////
-
-void FASTCALL VIDDummyVdp2SetPriorityNBG3(UNUSED int priority)
-{
-}
-
-//////////////////////////////////////////////////////////////////////////////
-
-void FASTCALL VIDDummyVdp2SetPriorityRBG0(UNUSED int priority)
-{
-}
-
-//////////////////////////////////////////////////////////////////////////////
-
-void VIDDummyOnScreenDebugMessage(UNUSED char * string, ...)
+void VIDDummyGetGlSize(int *width, int *height)
 {
+   *width = 0;
+   *height = 0;
 }
