@@ -25,72 +25,12 @@
 #include "vdp2.h"
 #include "debug.h"
 
-typedef struct
-{
-   int vertices[8];
-   int cellw, cellh;
-   int flipfunction;
-   int priority;
-   int dst;
-   /* The above fields MUST NOT BE CHANGED (including inserting new fields)
-    * unless YglSprite is also updated in ygl.h */
-
-   int cellw_bits, cellh_bits;
-   int mapwh;
-   int planew, planew_bits, planeh, planeh_bits;
-   int pagewh, pagewh_bits;
-   int patternwh, patternwh_bits;
-   int patterndatasize, patterndatasize_bits;
-   int specialfunction;
-   u32 addr, charaddr, paladdr;
-   int colornumber;
-   int isbitmap;
-   u16 supplementdata;
-   int auxmode;
-   int enable;
-   int x, y;
-   int alpha;
-   int coloroffset;
-   int transparencyenable;
-   int specialprimode;
-
-   s32 cor;
-   s32 cog;
-   s32 cob;
-
-   float coordincx, coordincy;
-   void FASTCALL (* PlaneAddr)(void *, int);
-   u32 FASTCALL (* PostPixelFetchCalc)(void *, u32);
-   int patternpixelwh;
-   int draww;
-   int drawh;
-   int rotatenum;
-   int rotatemode;
-   int mosaicxmask;
-   int mosaicymask;
-   int islinescroll;
-   u32 linescrolltbl;
-   int wctl;
-   int islinewindow;
-   int isverticalscroll;
-   u32 verticalscrolltbl;
-   int verticalscrollinc;
-   
-   // WindowMode
-   u8  LogicWin;    // Window Logic AND OR
-   u8  bEnWin0;     // Enable Window0
-   u8  bEnWin1;     // Enable Window1
-   u8  WindowArea0; // Window Area Mode 0
-   u8  WindowArea1; // Window Area Mode 1
-   
-} vdp2draw_struct;
-
 typedef struct 
 {
-   int  WinShowLine;
-	int WinHStart;
-	int WinHEnd;
-} vdp2WindowInfo;
+   short LineScrollValH;
+   short LineScrollValV;
+   int CoordinateIncH;
+} vdp2Lineinfo;
 
 typedef struct
 {
@@ -130,8 +70,119 @@ typedef struct
    float dY;
    int screenover;
    int msb;
+   
+   void FASTCALL (* PlaneAddr)(void *, int);
+   u32 charaddr;
+   int planew, planew_bits, planeh, planeh_bits;
+   int MaxH,MaxV;
+
+   float Xsp;
+   float Ysp;   
+   float dx;
+   float dy;
+   float lkx;
+   float lky;   
+   int KtablV;
+   int ShiftPaneX;
+   int ShiftPaneY;   
+   int MskH;
+   int MskV;
+   u32 lineaddr;
+   u32 PlaneAddrv[16];
+   
 } vdp2rotationparameter_struct;
 
+typedef struct 
+{
+   int  WinShowLine;
+    int WinHStart;
+    int WinHEnd;
+} vdp2WindowInfo;
+
+typedef u32 FASTCALL (*Vdp2ColorRamGetColor_func)(void *, u32 , int);
+typedef vdp2rotationparameter_struct * FASTCALL (*Vdp2GetRParam_func)(void *, int, int);
+
+typedef struct 
+{
+   int vertices[8];
+   int cellw, cellh;
+   int flipfunction;
+   int priority;
+   int dst;
+   int uclipmode;
+   int blendmode;
+   /* The above fields MUST NOT BE CHANGED (including inserting new fields)
+    * unless YglSprite is also updated in ygl.h */
+
+   int cellw_bits, cellh_bits;
+   int mapwh;
+   int planew, planew_bits, planeh, planeh_bits;
+   int pagewh, pagewh_bits;
+   int patternwh, patternwh_bits;
+   int patterndatasize, patterndatasize_bits;
+   int specialfunction;
+   int specialcolorfunction;
+   int specialcolormode;
+   int specialcode;
+   u32 addr, charaddr, paladdr;
+   int colornumber;
+   int isbitmap;
+   u16 supplementdata;
+   int auxmode;
+   int enable;
+   int x, y;
+   int sh,sv;
+   int alpha;
+   int coloroffset;
+   int transparencyenable;
+   int specialprimode;
+
+   s32 cor;
+   s32 cog;
+   s32 cob;
+
+   float coordincx, coordincy;
+   void FASTCALL (* PlaneAddr)(void *, int);
+   u32 FASTCALL (*Vdp2ColorRamGetColor)(void *, u32 , int );
+   u32 FASTCALL (*PostPixelFetchCalc)(void *, u32);
+   int patternpixelwh;
+   int draww;
+   int drawh;
+   int rotatenum;
+   int rotatemode;
+   int mosaicxmask;
+   int mosaicymask;
+   int islinescroll;
+   u32 linescrolltbl;
+   u32 lineinc;
+   vdp2Lineinfo * lineinfo;
+   int wctl;
+   int islinewindow;
+   int isverticalscroll;
+   u32 verticalscrolltbl;
+   int verticalscrollinc;
+   int linescreen;
+   
+   // WindowMode
+   u8  LogicWin;    // Window Logic AND OR
+   u8  bEnWin0;     // Enable Window0
+   u8  bEnWin1;     // Enable Window1
+   u8  WindowArea0; // Window Area Mode 0
+   u8  WindowArea1; // Window Area Mode 1
+   
+   // Rotate Screen
+   vdp2WindowInfo * pWinInfo;
+   int WindwAreaMode;
+   vdp2rotationparameter_struct * FASTCALL (*GetKValueA)(vdp2rotationparameter_struct*,int);
+   vdp2rotationparameter_struct * FASTCALL (*GetKValueB)(vdp2rotationparameter_struct*,int);   
+   vdp2rotationparameter_struct * FASTCALL (*GetRParam)(void *, int h,int v);
+   u32 LineColorBase;
+   
+   void (*LoadLineParams)(void *, int line);
+} vdp2draw_struct;
+
+
+
 #define FP_SIZE 16
 typedef s32 fixed32;
 
@@ -173,6 +224,9 @@ typedef struct
    fixed32 dY;
    int screenover;
    int msb;
+   int linescreen;
+
+   void FASTCALL (* PlaneAddr)(void *, int);
 } vdp2rotationparameterfp_struct;
 
 typedef struct
@@ -187,6 +241,7 @@ typedef struct
 #define tofloat(v) ((float)(v) / (float)(1 << FP_SIZE))
 #define mulfixed(a,b) ((fixed32)((s64)(a) * (s64)(b) >> FP_SIZE))
 #define divfixed(a,b) (((s64)(a) << FP_SIZE) / (b))
+#define decipart(v) (v & ((1 << FP_SIZE) - 1))
 
 void FASTCALL Vdp2NBG0PlaneAddr(vdp2draw_struct *info, int i);
 void FASTCALL Vdp2NBG1PlaneAddr(vdp2draw_struct *info, int i);
@@ -199,8 +254,6 @@ void FASTCALL Vdp2ParameterBPlaneAddr(vdp2draw_struct *info, int i);
 float Vdp2ReadCoefficientMode0_2(vdp2rotationparameter_struct *parameter, u32 addr);
 fixed32 Vdp2ReadCoefficientMode0_2FP(vdp2rotationparameterfp_struct *parameter, u32 addr);
 
-void VideoInitGlut(void);
-
 //////////////////////////////////////////////////////////////////////////////
 
 static INLINE int GenerateRotatedXPos(vdp2rotationparameter_struct *p, int x, int y)
@@ -353,6 +406,33 @@ static INLINE void ReadBitmapSize(vdp2draw_struct *info, u16 bm, int mask)
 
 //////////////////////////////////////////////////////////////////////////////
 
+static INLINE void ReadPlaneSizeR(vdp2rotationparameter_struct *info, u16 reg)
+{
+   switch(reg & 0x3)
+   {
+      case 0:
+         info->planew = info->planeh = 1;
+         info->planew_bits = info->planeh_bits = 0;
+         break;
+      case 1:
+         info->planew = 2; info->planew_bits = 1;
+         info->planeh = 1; info->planeh_bits = 0;
+         break;
+      case 3:
+         info->planew = info->planeh = 2;
+         info->planew_bits = info->planeh_bits = 1;
+         break;
+      default: // Not sure what 0x2 does, though a few games seem to use it
+         info->planew = info->planeh = 1;
+         info->planew_bits = info->planeh_bits = 0;
+         break;
+   }
+   
+
+}
+
+
+
 static INLINE void ReadPlaneSize(vdp2draw_struct *info, u16 reg)
 {
    switch(reg & 0x3)
@@ -424,7 +504,7 @@ static INLINE void ReadMosaicData(vdp2draw_struct *info, u16 mask)
       info->mosaicxmask = 1;
       info->mosaicymask = 1;
    }
-}
+} 
 
 //////////////////////////////////////////////////////////////////////////////
 
@@ -434,29 +514,80 @@ static INLINE void ReadLineScrollData(vdp2draw_struct *info, u16 mask, u32 tbl)
    {
       info->islinescroll = (mask >> 1) & 0x7;
       info->linescrolltbl = (tbl & 0x7FFFE) << 1;
+	   info->lineinc = 1 << ((mask >> 4) & 0x03);
    }
    else
+   {
       info->islinescroll = 0;
+	  info->lineinc = 0;
+   }
+}
+
+
+//////////////////////////////////////////////////////////////////////////////
+
+static INLINE void ReadWindowCoordinates(int num, clipping_struct * clip)
+{
+   if (num == 0)
+   {
+      // Window 0
+      clip->xstart = Vdp2Regs->WPSX0;
+      clip->ystart = Vdp2Regs->WPSY0 & 0x1FF;
+      clip->xend = Vdp2Regs->WPEX0;
+      clip->yend = Vdp2Regs->WPEY0 & 0x1FF;
+   }
+   else
+   {
+      // Window 1
+      clip->xstart = Vdp2Regs->WPSX1;
+      clip->ystart = Vdp2Regs->WPSY1 & 0x1FF;
+      clip->xend = Vdp2Regs->WPEX1;
+      clip->yend = Vdp2Regs->WPEY1 & 0x1FF;
+   }
+
+   switch ((Vdp2Regs->TVMD >> 1) & 0x3)
+   {
+      case 0: // Normal
+         clip->xstart = (clip->xstart >> 1) & 0x1FF;
+         clip->xend = (clip->xend >> 1) & 0x1FF;
+         break;
+      case 1: // Hi-Res
+         clip->xstart = clip->xstart & 0x3FF;
+         clip->xend = clip->xend & 0x3FF;
+         break;
+      case 2: // Exclusive Normal
+         clip->xstart = clip->xstart & 0x1FF;
+         clip->xend = clip->xend & 0x1FF;
+         break;
+      case 3: // Exclusive Hi-Res
+         clip->xstart = (clip->xstart & 0x3FF) >> 1;
+         clip->xend = (clip->xend & 0x3FF) >> 1;
+         break;
+   }
+
+   if ((Vdp2Regs->TVMD & 0xC0) == 0xC0)
+   {
+      // Double-density interlace
+      clip->ystart >>= 1;
+      clip->yend >>= 1;
+   }
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
 static INLINE void ReadWindowData(int wctl, clipping_struct *clip)
 {
+   clip[0].xstart = clip[0].ystart = clip[0].xend = clip[0].yend = 0;
+   clip[1].xstart = clip[1].ystart = clip[1].xend = clip[1].yend = 0;
+
    if (wctl & 0x2)
    {
-      clip[0].xstart = Vdp2Regs->WPSX0 >> 1; // fix me
-      clip[0].ystart = Vdp2Regs->WPSY0;
-      clip[0].xend = Vdp2Regs->WPEX0 >> 1; // fix me
-      clip[0].yend = Vdp2Regs->WPEY0;
+      ReadWindowCoordinates(0, clip);
    }
 
    if (wctl & 0x8)
    {
-      clip[1].xstart = Vdp2Regs->WPSX1 >> 1; // fix me
-      clip[1].ystart = Vdp2Regs->WPSY1;
-      clip[1].xend = Vdp2Regs->WPEX1 >> 1; // fix me
-      clip[1].yend = Vdp2Regs->WPEY1;
+      ReadWindowCoordinates(1, clip + 1);
    }
 
    if (wctl & 0x20)
@@ -485,27 +616,58 @@ static INLINE void ReadLineWindowData(int *islinewindow, int wctl, u32 *linewnd0
 
 //////////////////////////////////////////////////////////////////////////////
 
+static INLINE void ReadOneLineWindowClip(clipping_struct *clip, u32 *linewndaddr)
+{
+   clip->xstart = T1ReadWord(Vdp2Ram, *linewndaddr);
+   *linewndaddr += 2;
+   clip->xend = T1ReadWord(Vdp2Ram, *linewndaddr);
+   *linewndaddr += 2;
+
+   /* Ok... that looks insane... but there's at least two games (3D Baseball and
+   Panzer Dragoon Saga) that set the line window end to 0xFFFF and expect the line
+   window to be invalid for those lines... */
+   if (clip->xend == 0xFFFF)
+   {
+      clip->xstart = 0;
+      clip->xend = 0;
+      return;
+   }
+
+   clip->xstart &= 0x3FF;
+   clip->xend &= 0x3FF;
+
+   switch ((Vdp2Regs->TVMD >> 1) & 0x3)
+   {
+      case 0: // Normal
+         clip->xstart = (clip->xstart >> 1) & 0x1FF;
+         clip->xend = (clip->xend >> 1) & 0x1FF;
+         break;
+      case 1: // Hi-Res
+         clip->xstart = clip->xstart & 0x3FF;
+         clip->xend = clip->xend & 0x3FF;
+         break;
+      case 2: // Exclusive Normal
+         clip->xstart = clip->xstart & 0x1FF;
+         clip->xend = clip->xend & 0x1FF;
+         break;
+      case 3: // Exclusive Hi-Res
+         clip->xstart = (clip->xstart & 0x3FF) >> 1;
+         clip->xend = (clip->xend & 0x3FF) >> 1;
+         break;
+   }
+}
+
 static INLINE void ReadLineWindowClip(int islinewindow, clipping_struct *clip, u32 *linewnd0addr, u32 *linewnd1addr)
 {
    if (islinewindow)
    {
       // Fetch new xstart and xend values from table
       if (islinewindow & 0x1)
-      {
          // Window 0
-         clip[0].xstart = (T1ReadWord(Vdp2Ram, linewnd0addr[0]) & 0x3FF) >> 1; // fix me
-         linewnd0addr[0]+=2;
-         clip[0].xend = (T1ReadWord(Vdp2Ram, linewnd0addr[0]) & 0x3FF) >> 1; // fix me
-         linewnd0addr[0]+=2;
-      }
+         ReadOneLineWindowClip(clip, linewnd0addr);
       if (islinewindow & 0x2)
-      {
          // Window 1
-         clip[1].xstart = (T1ReadWord(Vdp2Ram, linewnd1addr[0]) & 0x3FF) >> 1; // fix me
-         linewnd1addr[0]+=2;
-         clip[1].xend = (T1ReadWord(Vdp2Ram, linewnd1addr[0]) & 0x3FF) >> 1; // fix me
-         linewnd1addr[0]+=2;
-      }
+         ReadOneLineWindowClip(clip + 1, linewnd1addr);
    }
 }
 
@@ -606,6 +768,7 @@ static INLINE void Vdp2ReadCoefficientFP(vdp2rotationparameterfp_struct *paramet
          {
             i = T1ReadLong(Vdp2Ram, addr);
             parameter->msb = (i >> 31) & 0x1;
+            parameter->linescreen = (i >> 24) & 0x7F;
             parameter->Xp = (signed) ((i & 0x007FFFFF) | (i & 0x00800000 ? 0xFF800000 : 0x00000000)) * 256;
          }
 
@@ -616,138 +779,184 @@ static INLINE void Vdp2ReadCoefficientFP(vdp2rotationparameterfp_struct *paramet
 
 //////////////////////////////////////////////////////////////////////////////
 
-static INLINE void Vdp1ProcessSpritePixel(int type, u16 *pixel, int *shadow, int *priority, int *colorcalc)
+typedef struct {
+    int normalshadow;
+    int msbshadow;
+    int priority;
+    int colorcalc;
+} spritepixelinfo_struct;
+
+static INLINE void Vdp1GetSpritePixelInfo(int type, u16 * pixel, spritepixelinfo_struct * spi)
 {
+   memset(spi, 0, sizeof(spritepixelinfo_struct));
+
    switch(type)
    {
       case 0x0:
       {
          // Type 0(2-bit priority, 3-bit color calculation, 11-bit color data)
-         *priority = *pixel >> 14;
-         *colorcalc = (*pixel >> 11) & 0x7;
+         spi->priority = *pixel >> 14;
+         spi->colorcalc = (*pixel >> 11) & 0x7;
          *pixel = *pixel & 0x7FF;
+         spi->normalshadow = (*pixel == 0x7FE);
          break;
       }
       case 0x1:
       {
          // Type 1(3-bit priority, 2-bit color calculation, 11-bit color data)
-         *priority = *pixel >> 13;
-         *colorcalc = (*pixel >> 11) & 0x3;
+         spi->priority = *pixel >> 13;
+         spi->colorcalc = (*pixel >> 11) & 0x3;
          *pixel &= 0x7FF;
+         spi->normalshadow = (*pixel == 0x7FE);
          break;
       }
       case 0x2:
       {
          // Type 2(1-bit shadow, 1-bit priority, 3-bit color calculation, 11-bit color data)
-         *shadow = *pixel >> 15;
-         *priority = (*pixel >> 14) & 0x1;
-         *colorcalc = (*pixel >> 11) & 0x7;
+         spi->msbshadow = *pixel >> 15;
+         spi->priority = (*pixel >> 14) & 0x1;
+         spi->colorcalc = (*pixel >> 11) & 0x7;
          *pixel &= 0x7FF;
+         spi->normalshadow = (*pixel == 0x7FE);
          break;
       }
       case 0x3:
       {
          // Type 3(1-bit shadow, 2-bit priority, 2-bit color calculation, 11-bit color data)
-         *shadow = *pixel >> 15;
-         *priority = (*pixel >> 13) & 0x3;
-         *colorcalc = (*pixel >> 11) & 0x3;
+         spi->msbshadow = *pixel >> 15;
+         spi->priority = (*pixel >> 13) & 0x3;
+         spi->colorcalc = (*pixel >> 11) & 0x3;
          *pixel &= 0x7FF;
+         spi->normalshadow = (*pixel == 0x7FE);
          break;
       }
       case 0x4:
       {
          // Type 4(1-bit shadow, 2-bit priority, 3-bit color calculation, 10-bit color data)
-         *shadow = *pixel >> 15;
-         *priority = (*pixel >> 13) & 0x3;
-         *colorcalc = (*pixel >> 10) & 0x7;
+         spi->msbshadow = *pixel >> 15;
+         spi->priority = (*pixel >> 13) & 0x3;
+         spi->colorcalc = (*pixel >> 10) & 0x7;
          *pixel &= 0x3FF;
+         spi->normalshadow = (*pixel == 0x3FE);
          break;
       }
       case 0x5:
       {
          // Type 5(1-bit shadow, 3-bit priority, 1-bit color calculation, 11-bit color data)
-         *shadow = *pixel >> 15;
-         *priority = (*pixel >> 12) & 0x7;
-         *colorcalc = (*pixel >> 11) & 0x1;
+         spi->msbshadow = *pixel >> 15;
+         spi->priority = (*pixel >> 12) & 0x7;
+         spi->colorcalc = (*pixel >> 11) & 0x1;
          *pixel &= 0x7FF;
+         spi->normalshadow = (*pixel == 0x7FE);
          break;
       }
       case 0x6:
       {
          // Type 6(1-bit shadow, 3-bit priority, 2-bit color calculation, 10-bit color data)
-         *shadow = *pixel >> 15;
-         *priority = (*pixel >> 12) & 0x7;
-         *colorcalc = (*pixel >> 10) & 0x3;
+         spi->msbshadow = *pixel >> 15;
+         spi->priority = (*pixel >> 12) & 0x7;
+         spi->colorcalc = (*pixel >> 10) & 0x3;
          *pixel &= 0x3FF;
+         spi->normalshadow = (*pixel == 0x3FE);
          break;
       }
       case 0x7:
       {
          // Type 7(1-bit shadow, 3-bit priority, 3-bit color calculation, 9-bit color data)
-         *shadow = *pixel >> 15;
-         *priority = (*pixel >> 12) & 0x7;
-         *colorcalc = (*pixel >> 9) & 0x7;
+         spi->msbshadow = *pixel >> 15;
+         spi->priority = (*pixel >> 12) & 0x7;
+         spi->colorcalc = (*pixel >> 9) & 0x7;
          *pixel &= 0x1FF;
+         spi->normalshadow = (*pixel == 0x1FE);
          break;
       }
       case 0x8:
       {
          // Type 8(1-bit priority, 7-bit color data)
-         *priority = *pixel >> 7;
+         spi->priority = (*pixel >> 7) & 0x1;
          *pixel &= 0x7F;
+         spi->normalshadow = (*pixel == 0x7E);
          break;
       }
       case 0x9:
       {
          // Type 9(1-bit priority, 1-bit color calculation, 6-bit color data)
-         *priority = *pixel >> 7;
-         *colorcalc = (*pixel >> 6) & 0x1;
+         spi->priority = (*pixel >> 7) & 0x1;
+         spi->colorcalc = (*pixel >> 6) & 0x1;
          *pixel &= 0x3F;
+         spi->normalshadow = (*pixel == 0x3E);
          break;
       }
       case 0xA:
       {
          // Type A(2-bit priority, 6-bit color data)
-         *priority = *pixel >> 6;
+         spi->priority = (*pixel >> 6) & 0x3;
          *pixel &= 0x3F;
+         spi->normalshadow = (*pixel == 0x3E);
          break;
       }
       case 0xB:
       {
          // Type B(2-bit color calculation, 6-bit color data)
-         *colorcalc = *pixel >> 6;
+         spi->colorcalc = (*pixel >> 6) & 0x3;
          *pixel &= 0x3F;
+         spi->normalshadow = (*pixel == 0x3E);
          break;
       }
       case 0xC:
       {
          // Type C(1-bit special priority, 8-bit color data - bit 7 is shared)
-         *priority = *pixel >> 7;
+         spi->priority = (*pixel >> 7) & 0x1;
+         spi->normalshadow = (*pixel == 0xFE);
          break;
       }
       case 0xD:
       {
          // Type D(1-bit special priority, 1-bit special color calculation, 8-bit color data - bits 6 and 7 are shared)
-         *priority = *pixel >> 7;
-         *colorcalc = (*pixel >> 6) & 0x1;
+         spi->priority = (*pixel >> 7) & 0x1;
+         spi->colorcalc = (*pixel >> 6) & 0x1;
+         spi->normalshadow = (*pixel == 0xFE);
          break;
       }
       case 0xE:
       {
          // Type E(2-bit special priority, 8-bit color data - bits 6 and 7 are shared)
-         *priority = *pixel >> 6;
+         spi->priority = (*pixel >> 6) & 0x3;
+         spi->normalshadow = (*pixel == 0xFE);
          break;
       }
       case 0xF:
       {
          // Type F(2-bit special color calculation, 8-bit color data - bits 6 and 7 are shared)
-         *colorcalc = *pixel >> 6;
+         spi->colorcalc = (*pixel >> 6) & 0x3;
+         spi->normalshadow = (*pixel == 0xFE);
          break;
       }
       default: break;
    }
 }
 
+//////////////////////////////////////////////////////////////////////////////
+
+static INLINE void Vdp1ProcessSpritePixel(int type, u16 *pixel, int *shadow, int *priority, int *colorcalc)
+{
+   spritepixelinfo_struct spi;
+
+   Vdp1GetSpritePixelInfo(type, pixel, &spi);
+   *shadow = spi.msbshadow;
+   *priority = spi.priority;
+   *colorcalc = spi.colorcalc;
+}
+
+#define VDPLINE_SZ(a) ((a)&0x04)
+#define VDPLINE_SY(a) ((a)&0x02)
+#define VDPLINE_SX(a) ((a)&0x01)
+#define OVERMODE_REPEAT      0
+#define OVERMODE_SELPATNAME  1
+#define OVERMODE_TRANSE      2
+#define OVERMODE_512         3
+
+
 //////////////////////////////////////////////////////////////////////////////
 
 #endif
