@@ -37,7 +37,6 @@
 #include "yui.h"
 
 #include <stdlib.h>
-#include <stdarg.h>
 #include <limits.h>
 
 #if defined WORDS_BIGENDIAN
@@ -95,7 +94,6 @@ void FASTCALL VIDSoftVdp2SetPriorityNBG1(int priority);
 void FASTCALL VIDSoftVdp2SetPriorityNBG2(int priority);
 void FASTCALL VIDSoftVdp2SetPriorityNBG3(int priority);
 void FASTCALL VIDSoftVdp2SetPriorityRBG0(int priority);
-void VIDSoftOnScreenDebugMessage(char *string, ...);
 void VIDSoftGetGlSize(int *width, int *height);
 void VIDSoftVdp1SwapFrameBuffer(void);
 void VIDSoftVdp1EraseFrameBuffer(void);
@@ -127,17 +125,10 @@ VIDSoftVdp2Reset,
 VIDSoftVdp2DrawStart,
 VIDSoftVdp2DrawEnd,
 VIDSoftVdp2DrawScreens,
-VIDSoftVdp2SetResolution,
-VIDSoftVdp2SetPriorityNBG0,
-VIDSoftVdp2SetPriorityNBG1,
-VIDSoftVdp2SetPriorityNBG2,
-VIDSoftVdp2SetPriorityNBG3,
-VIDSoftVdp2SetPriorityRBG0,
-VIDSoftOnScreenDebugMessage,
 VIDSoftGetGlSize,
 };
 
-u32 *dispbuffer=NULL;
+pixel_t *dispbuffer=NULL;
 u8 *vdp1framebuffer[2]= { NULL, NULL };
 u8 *vdp1frontframebuffer;
 u8 *vdp1backframebuffer;
@@ -165,9 +156,6 @@ static int outputheight;
 static int resxratio;
 static int resyratio;
 
-static char message[512];
-static int msglength;
-
 typedef struct { s16 x; s16 y; } vdp1vertex;
 
 typedef struct
@@ -805,6 +793,14 @@ static void FASTCALL Vdp2DrawRotationFP(vdp2draw_struct *info, vdp2rotationparam
    int x, y;
    screeninfo_struct sinfo;
    vdp2rotationparameterfp_struct *p=&parameter[info->rotatenum];
+   clipping_struct clip[2];
+   u32 linewnd0addr, linewnd1addr;
+
+   clip[0].xstart = clip[0].ystart = clip[0].xend = clip[0].yend = 0;
+   clip[1].xstart = clip[1].ystart = clip[1].xend = clip[1].yend = 0;
+   ReadWindowData(info->wctl, clip);
+   linewnd0addr = linewnd1addr = 0;
+   ReadLineWindowData(&info->islinewindow, info->wctl, &linewnd0addr, &linewnd1addr);
 
    Vdp2ReadRotationTableFP(info->rotatenum, p);
 
@@ -833,11 +829,15 @@ static void FASTCALL Vdp2DrawRotationFP(vdp2draw_struct *info, vdp2rotationparam
          for (j = 0; j < vdp2height; j++)
          {
             info->LoadLineParams(info, j);
+            ReadLineWindowClip(info->islinewindow, clip, &linewnd0addr, &linewnd1addr);
 
             for (i = 0; i < vdp2width; i++)
             {
                u32 color;
 
+               if (!TestBothWindow(info->wctl, clip, i, j))
+                  continue;
+
                x = GenerateRotatedXPosFP(p, i, xmul, ymul, C) & sinfo.xmask;
                y = GenerateRotatedYPosFP(p, i, xmul, ymul, F) & sinfo.ymask;
 
@@ -879,16 +879,16 @@ static void FASTCALL Vdp2DrawRotationFP(vdp2draw_struct *info, vdp2rotationparam
 
       clipping_struct rpwindow[2];
       int userpwindow = 0;
-      int islinewindow = 0;
-      u32 linewnd0addr, linewnd1addr;
+      int isrplinewindow = 0;
+      u32 rplinewnd0addr, rplinewnd1addr;
 
       if ((Vdp2Regs->RPMD & 3) == 2)
          p2 = &parameter[1 - info->rotatenum];
       else if ((Vdp2Regs->RPMD & 3) == 3)
       {
          ReadWindowData(Vdp2Regs->WCTLD, rpwindow);
-         linewnd0addr = linewnd1addr = 0;
-         ReadLineWindowData(&islinewindow, Vdp2Regs->WCTLD, &linewnd0addr, &linewnd1addr);
+         rplinewnd0addr = rplinewnd1addr = 0;
+         ReadLineWindowData(&isrplinewindow, Vdp2Regs->WCTLD, &rplinewnd0addr, &rplinewnd1addr);
          userpwindow = 1;
          p2 = &parameter[1 - info->rotatenum];
       }
@@ -953,9 +953,10 @@ static void FASTCALL Vdp2DrawRotationFP(vdp2draw_struct *info, vdp2rotationparam
          }
 
          info->LoadLineParams(info, j);
+         ReadLineWindowClip(info->islinewindow, clip, &linewnd0addr, &linewnd1addr);
 
          if (userpwindow)
-            ReadLineWindowClip(islinewindow, rpwindow, &linewnd0addr, &linewnd1addr);
+            ReadLineWindowClip(isrplinewindow, rpwindow, &rplinewnd0addr, &rplinewnd1addr);
 
          for (i = 0; i < vdp2width; i++)
          {
@@ -980,12 +981,32 @@ static void FASTCALL Vdp2DrawRotationFP(vdp2draw_struct *info, vdp2rotationparam
                rcoefx2 += decipart(p2->deltaKAx);
             }
 
+            if (!TestBothWindow(info->wctl, clip, i, j))
+               continue;
+
             if (((! userpwindow) && p->msb) || (userpwindow && (! TestBothWindow(Vdp2Regs->WCTLD, rpwindow, i, j))))
             {
                if ((p2 == NULL) || (p2->coefenab && p2->msb)) continue;
 
-               x = GenerateRotatedXPosFP(p2, i, xmul2, ymul2, C2) & sinfo2.xmask;
-               y = GenerateRotatedYPosFP(p2, i, xmul2, ymul2, F2) & sinfo2.ymask;
+               x = GenerateRotatedXPosFP(p2, i, xmul2, ymul2, C2);
+               y = GenerateRotatedYPosFP(p2, i, xmul2, ymul2, F2);
+
+               switch(p2->screenover) {
+                  case 0:
+                     x &= sinfo2.xmask;
+                     y &= sinfo2.ymask;
+                     break;
+                  case 1:
+                     VDP2LOG("Screen-over mode 1 not implemented");
+                     x &= sinfo2.xmask;
+                     y &= sinfo2.ymask;
+                     break;
+                  case 2:
+                     if ((x > sinfo2.xmask) || (y > sinfo2.ymask)) continue;
+                     break;
+                  case 3:
+                     if ((x > 512) || (y > 512)) continue;
+               }
 
                // Convert coordinates into graphics
                if (!info->isbitmap)
@@ -997,8 +1018,25 @@ static void FASTCALL Vdp2DrawRotationFP(vdp2draw_struct *info, vdp2rotationparam
             else if (p->msb) continue;
             else
             {
-               x = GenerateRotatedXPosFP(p, i, xmul, ymul, C) & sinfo.xmask;
-               y = GenerateRotatedYPosFP(p, i, xmul, ymul, F) & sinfo.ymask;
+               x = GenerateRotatedXPosFP(p, i, xmul, ymul, C);
+               y = GenerateRotatedYPosFP(p, i, xmul, ymul, F);
+
+               switch(p->screenover) {
+                  case 0:
+                     x &= sinfo.xmask;
+                     y &= sinfo.ymask;
+                     break;
+                  case 1:
+                     VDP2LOG("Screen-over mode 1 not implemented");
+                     x &= sinfo.xmask;
+                     y &= sinfo.ymask;
+                     break;
+                  case 2:
+                     if ((x > sinfo.xmask) || (y > sinfo.ymask)) continue;
+                     break;
+                  case 3:
+                     if ((x > 512) || (y > 512)) continue;
+               }
 
                // Convert coordinates into graphics
                if (!info->isbitmap)
@@ -1230,6 +1268,7 @@ static void Vdp2DrawNBG0(void)
    else
       info.alpha = 0x3F;
    if ((Vdp2Regs->CCCTL & 0x201) == 0x201) info.alpha |= 0x80;
+   else if ((Vdp2Regs->CCCTL & 0x101) == 0x101) info.alpha |= 0x80;
    info.specialcolormode = Vdp2Regs->SFCCMD & 0x3;
    if (Vdp2Regs->SFSEL & 0x1)
       info.specialcode = Vdp2Regs->SFCODE >> 8;
@@ -1329,6 +1368,7 @@ static void Vdp2DrawNBG1(void)
    else
       info.alpha = 0x3F;
    if ((Vdp2Regs->CCCTL & 0x202) == 0x202) info.alpha |= 0x80;
+   else if ((Vdp2Regs->CCCTL & 0x102) == 0x102) info.alpha |= 0x80;
    info.specialcolormode = (Vdp2Regs->SFCCMD >> 2) & 0x3;
    if (Vdp2Regs->SFSEL & 0x2)
       info.specialcode = Vdp2Regs->SFCODE >> 8;
@@ -1346,7 +1386,8 @@ static void Vdp2DrawNBG1(void)
    info.priority = nbg1priority;
    info.PlaneAddr = (void FASTCALL (*)(void *, int))&Vdp2NBG1PlaneAddr;
 
-   if (!(info.enable & Vdp2External.disptoggle))
+   if (!(info.enable & Vdp2External.disptoggle) ||
+       (Vdp2Regs->BGON & 0x1 && (Vdp2Regs->CHCTLA & 0x70) >> 4 == 4)) // If NBG0 16M mode is enabled, don't draw
       return;
 
    ReadMosaicData(&info, 0x2);
@@ -1409,6 +1450,7 @@ static void Vdp2DrawNBG2(void)
    else
       info.alpha = 0x3F;
    if ((Vdp2Regs->CCCTL & 0x204) == 0x204) info.alpha |= 0x80;
+   else if ((Vdp2Regs->CCCTL & 0x104) == 0x104) info.alpha |= 0x80;
    info.specialcolormode = (Vdp2Regs->SFCCMD >> 4) & 0x3;
    if (Vdp2Regs->SFSEL & 0x4)
       info.specialcode = Vdp2Regs->SFCODE >> 8;
@@ -1425,7 +1467,8 @@ static void Vdp2DrawNBG2(void)
    info.priority = nbg2priority;
    info.PlaneAddr = (void FASTCALL (*)(void *, int))&Vdp2NBG2PlaneAddr;
 
-   if (!(info.enable & Vdp2External.disptoggle))
+   if (!(info.enable & Vdp2External.disptoggle) ||
+      (Vdp2Regs->BGON & 0x1 && (Vdp2Regs->CHCTLA & 0x70) >> 4 >= 2)) // If NBG0 2048/32786/16M mode is enabled, don't draw
       return;
 
    ReadMosaicData(&info, 0x4);
@@ -1475,6 +1518,7 @@ static void Vdp2DrawNBG3(void)
    else
       info.alpha = 0x3F;
    if ((Vdp2Regs->CCCTL & 0x208) == 0x208) info.alpha |= 0x80;
+   else if ((Vdp2Regs->CCCTL & 0x108) == 0x108) info.alpha |= 0x80;
    info.specialcolormode = (Vdp2Regs->SFCCMD >> 6) & 0x3;
    if (Vdp2Regs->SFSEL & 0x8)
       info.specialcode = Vdp2Regs->SFCODE >> 8;
@@ -1491,7 +1535,9 @@ static void Vdp2DrawNBG3(void)
    info.priority = nbg3priority;
    info.PlaneAddr = (void FASTCALL (*)(void *, int))&Vdp2NBG3PlaneAddr;
 
-   if (!(info.enable & Vdp2External.disptoggle))
+   if (!(info.enable & Vdp2External.disptoggle) ||
+      (Vdp2Regs->BGON & 0x1 && (Vdp2Regs->CHCTLA & 0x70) >> 4 == 4) || // If NBG0 16M mode is enabled, don't draw
+      (Vdp2Regs->BGON & 0x2 && (Vdp2Regs->CHCTLA & 0x3000) >> 12 >= 2)) // If NBG1 2048/32786 is enabled, don't draw
       return;
 
    ReadMosaicData(&info, 0x8);
@@ -1601,6 +1647,7 @@ static void Vdp2DrawRBG0(void)
    else
       info.alpha = 0x3F;
    if ((Vdp2Regs->CCCTL & 0x210) == 0x210) info.alpha |= 0x80;
+   else if ((Vdp2Regs->CCCTL & 0x110) == 0x110) info.alpha |= 0x80;
    info.specialcolormode = (Vdp2Regs->SFCCMD >> 8) & 0x3;
    if (Vdp2Regs->SFSEL & 0x10)
       info.specialcode = Vdp2Regs->SFCODE >> 8;
@@ -1643,7 +1690,7 @@ int VIDSoftInit(void)
    if (TitanInit() == -1)
       return -1;
 
-   if ((dispbuffer = (u32 *)calloc(sizeof(u32), 704 * 512)) == NULL)
+   if ((dispbuffer = (pixel_t *)calloc(sizeof(pixel_t), 704 * 512)) == NULL)
       return -1;
 
    // Initialize VDP1 framebuffer 1
@@ -1660,20 +1707,6 @@ int VIDSoftInit(void)
    vdp2height = 224;
 
 #ifdef USE_OPENGL
-   YuiSetVideoAttribute(DOUBLEBUFFER, 1);
-
-   if (!YglScreenInit(8, 8, 8, 24))
-   {
-      if (!YglScreenInit(4, 4, 4, 24))
-      {
-         if (!YglScreenInit(5, 6, 5, 16))
-         {
-	    YuiErrorMsg("Couldn't set GL mode\n");
-            return -1;
-         }
-      }
-   }
-
    glClear(GL_COLOR_BUFFER_BIT);
 
    glMatrixMode(GL_PROJECTION);
@@ -1685,7 +1718,6 @@ int VIDSoftInit(void)
    glOrtho(-320, 320, -224, 224, 1, 0);
    outputwidth = 320;
    outputheight = 224;
-   msglength = 0;
 #endif
 
    return 0;
@@ -1717,11 +1749,6 @@ void VIDSoftResize(unsigned int w, unsigned int h, int on)
 #ifdef USE_OPENGL
    IsFullscreen = on;
 
-   if (on)
-      YuiSetVideoMode(w, h, 32, 1);
-   else
-      YuiSetVideoMode(w, h, 32, 0);
-
    glClear(GL_COLOR_BUFFER_BIT);
 
    glMatrixMode(GL_PROJECTION);
@@ -1730,7 +1757,7 @@ void VIDSoftResize(unsigned int w, unsigned int h, int on)
 
    glMatrixMode(GL_TEXTURE);
    glLoadIdentity();
-   glOrtho(-w, w, -h, h, 1, 0);
+   glOrtho(-(signed)w, w, -(signed)h, h, 1, 0);
 
    glViewport(0, 0, w, h);
    outputwidth = w;
@@ -2044,10 +2071,13 @@ static void putpixel8(int x, int y) {
 
 static void putpixel(int x, int y) {
 
-	u16* iPix = &((u16 *)vdp1backframebuffer)[(y * vdp1width) + x];
+	u16* iPix;
 	int mesh = cmd.CMDPMOD & 0x0100;
 	int SPD = ((cmd.CMDPMOD & 0x40) != 0);//show the actual color of transparent pixels if 1 (they won't be drawn transparent)
 
+	y /= vdp1interlace;
+	iPix = &((u16 *)vdp1backframebuffer)[(y * vdp1width) + x];
+
 	if (iPix >= (u16*) (vdp1backframebuffer + 0x40000))
 		return;
 
@@ -2085,8 +2115,9 @@ static void putpixel(int x, int y) {
 			if (!((currentPixel == 0) && !SPD)) 
 				*(iPix) = currentPixel;
 			break;
-		case 1: // shadow, TODO
-			*(iPix) = currentPixel;
+		case 1: // shadow
+			if (*(iPix) & (1 << 15)) // only if MSB of framebuffer data is set
+				*(iPix) = alphablend16(*(iPix), 0, (1 << 7)) | (1 << 15);
 			break;
 		case 2: // half luminance
 			*(iPix) = ((currentPixel & ~0x8421) >> 1) | (1 << 15);
@@ -2248,8 +2279,6 @@ static int DrawLineCallback(int x, int y, int i, void *data)
 		putpixel8(x, y);
     }
 
-	linedata->previousStep = currentPixel;
-
 	if (linedata->endcodesdetected == 2) return -1;
 
 	return 0;
@@ -2785,6 +2814,11 @@ int VIDSoftVdp2Reset(void)
 
 void VIDSoftVdp2DrawStart(void)
 {
+   int titanblendmode = TITAN_BLEND_TOP;
+   if (Vdp2Regs->CCCTL & 0x100) titanblendmode = TITAN_BLEND_ADD;
+   else if (Vdp2Regs->CCCTL & 0x200) titanblendmode = TITAN_BLEND_BOTTOM;
+   TitanSetBlendingMode(titanblendmode);
+
    Vdp2DrawBackScreen();
    Vdp2DrawLineScreen();
 }
@@ -2859,7 +2893,7 @@ void VIDSoftVdp2DrawEnd(void)
          for (i = 0; i < vdp2width; i++)
          {
             // See if screen position is clipped, if it isn't, continue
-            if (!TestBothWindow(wctl, clip, i, i2))
+            if (!TestBothWindow(wctl, clip, i * resxratio, i2))
             {
                continue;
             }
@@ -2884,18 +2918,24 @@ void VIDSoftVdp2DrawEnd(void)
                else if (pixel & 0x8000 && colormode)
                {
                   // 16 BPP               
-                     // if pixel is 0x8000, only draw pixel if sprite window
-                     // is disabled/sprite type 2-7. sprite types 0 and 1 are
-                     // -always- drawn and sprite types 8-F are always
-                     // transparent.
-                     if (pixel != 0x8000 || vdp1spritetype < 2 || (vdp1spritetype < 8 && !(Vdp2Regs->SPCTL & 0x10)))
-                        TitanPutPixel(prioritytable[0], i, i2, info.PostPixelFetchCalc(&info, COLSAT2YAB16(0x3F, pixel)), 0);
+                  u8 alpha = 0x3F;
+                  if ((SPCCCS == 3) && TestBothWindow(Vdp2Regs->WCTLD >> 8, colorcalcwindow, i, i2) && (Vdp2Regs->CCCTL & 0x40))
+                  {
+                     alpha = colorcalctable[0];
+                     if (Vdp2Regs->CCCTL & 0x300) alpha |= 0x80;
+                  }
+                  // if pixel is 0x8000, only draw pixel if sprite window
+                  // is disabled/sprite type 2-7. sprite types 0 and 1 are
+                  // -always- drawn and sprite types 8-F are always
+                  // transparent.
+                  if (pixel != 0x8000 || vdp1spritetype < 2 || (vdp1spritetype < 8 && !(Vdp2Regs->SPCTL & 0x10)))
+                     TitanPutPixel(prioritytable[0], i, i2, info.PostPixelFetchCalc(&info, COLSAT2YAB16(alpha, pixel)), 0);
                }
                else
                {
                   // Color bank
                   spritepixelinfo_struct spi;
-                  u8 alpha = 0xFF;
+                  u8 alpha = 0x3F;
                   u32 dot;
 
                   Vdp1GetSpritePixelInfo(vdp1spritetype, &pixel, &spi);
@@ -2924,29 +2964,40 @@ void VIDSoftVdp2DrawEnd(void)
 
                   if (TestBothWindow(Vdp2Regs->WCTLD >> 8, colorcalcwindow, i, i2) && (Vdp2Regs->CCCTL & 0x40))
                   {
+                     int transparent = 0;
+
                      /* Sprite color calculation */
                      switch(SPCCCS) {
                         case 0:
-                           if (prioritytable[spi.priority] <= SPCCN) {
-                              alpha = colorcalctable[spi.colorcalc];
-                           }
+                           if (prioritytable[spi.priority] <= SPCCN)
+                              transparent = 1;
                            break;
                         case 1:
-                           if (prioritytable[spi.priority] == SPCCN) {
-                              alpha = colorcalctable[spi.colorcalc];
-                           }
+                           if (prioritytable[spi.priority] == SPCCN)
+                              transparent = 1;
                            break;
                         case 2:
-                           if (prioritytable[spi.priority] >= SPCCN) {
-                              alpha = colorcalctable[spi.colorcalc];
-                           }
+                           if (prioritytable[spi.priority] >= SPCCN)
+                              transparent = 1;
                            break;
                         case 3:
-                           if (dot & 0x80000000) {
-                              alpha = colorcalctable[spi.colorcalc];
-                           }
+                           if (dot & 0x80000000)
+                              transparent = 1;
                            break;
                      }
+
+                     if (Vdp2Regs->CCCTL & 0x200) {
+                        /* "bottom" mode, the alpha channel will be used by another layer,
+                        so we set it regardless of whether sprites are transparent or not.
+                        The highest priority bit is only set if the sprite is transparent
+                        (in this case, it's the alpha channel of the lower priority layer
+                        that will be used. */
+                        alpha = colorcalctable[spi.colorcalc];
+                        if (transparent) alpha |= 0x80;
+                     } else if (transparent) {
+                        alpha = colorcalctable[spi.colorcalc];
+                        if (Vdp2Regs->CCCTL & 0x100) alpha |= 0x80;
+                     }
                   }
 
                   TitanPutPixel(prioritytable[spi.priority], i, i2, info.PostPixelFetchCalc(&info, COLSAT2YAB32(alpha, dot)), 0);
@@ -2961,7 +3012,7 @@ void VIDSoftVdp2DrawEnd(void)
                {
                   // Color bank(fix me)
                   spritepixelinfo_struct spi;
-                  u8 alpha = 0xFF;
+                  u8 alpha = 0x3F;
                   u32 dot;
 
                   Vdp1GetSpritePixelInfo(vdp1spritetype, &pixel, &spi);
@@ -2975,29 +3026,40 @@ void VIDSoftVdp2DrawEnd(void)
 
                   if (TestBothWindow(Vdp2Regs->WCTLD >> 8, colorcalcwindow, i, i2) && (Vdp2Regs->CCCTL & 0x40))
                   {
+                     int transparent = 0;
+
                      /* Sprite color calculation */
                      switch(SPCCCS) {
                         case 0:
-                           if (prioritytable[spi.priority] <= SPCCN) {
-                              alpha = colorcalctable[spi.colorcalc];
-                           }
+                           if (prioritytable[spi.priority] <= SPCCN)
+                              transparent = 1;
                            break;
                         case 1:
-                           if (prioritytable[spi.priority] == SPCCN) {
-                              alpha = colorcalctable[spi.colorcalc];
-                           }
+                           if (prioritytable[spi.priority] == SPCCN)
+                              transparent = 1;
                            break;
                         case 2:
-                           if (prioritytable[spi.priority] >= SPCCN) {
-                              alpha = colorcalctable[spi.colorcalc];
-                           }
+                           if (prioritytable[spi.priority] >= SPCCN)
+                              transparent = 1;
                            break;
                         case 3:
-                           if (dot & 0x80000000) {
-                              alpha = colorcalctable[spi.colorcalc];
-                           }
+                           if (dot & 0x80000000)
+                              transparent = 1;
                            break;
                      }
+
+                     if (Vdp2Regs->CCCTL & 0x200) {
+                        /* "bottom" mode, the alpha channel will be used by another layer,
+                        so we set it regardless of whether sprites are transparent or not.
+                        The highest priority bit is only set if the sprite is transparent
+                        (in this case, it's the alpha channel of the lower priority layer
+                        that will be used. */
+                        alpha = colorcalctable[spi.colorcalc];
+                        if (transparent) alpha |= 0x80;
+                     } else if (transparent) {
+                        alpha = colorcalctable[spi.colorcalc];
+                        if (Vdp2Regs->CCCTL & 0x100) alpha |= 0x80;
+                     }
                   }
 
                   TitanPutPixel(prioritytable[spi.priority], i, i2, info.PostPixelFetchCalc(&info, COLSAT2YAB32(alpha, dot)), 0);
@@ -3006,21 +3068,21 @@ void VIDSoftVdp2DrawEnd(void)
          }
       }
    }
-   {
-      int titanblendmode = TITAN_BLEND_TOP;
-      if (Vdp2Regs->CCCTL & 0x100) titanblendmode = TITAN_BLEND_ADD;
-      else if (Vdp2Regs->CCCTL & 0x200) titanblendmode = TITAN_BLEND_BOTTOM;
-      TitanRender(dispbuffer, titanblendmode);
-   }
+   TitanRender(dispbuffer);
 
    VIDSoftVdp1SwapFrameBuffer();
 
    if (OSDUseBuffer())
       OSDDisplayMessages(dispbuffer, vdp2width, vdp2height);
 
-#ifdef USE_OPENGL
-   glRasterPos2i(0, 0);
-   glPixelZoom((float)outputwidth / (float)vdp2width, 0 - ((float)outputheight / (float)vdp2height));
+#ifdef USE_OPENGL	
+	if (vdp2height == 224)
+		i = 8;
+	else
+		i = 0;
+
+   glRasterPos2i(0, outputheight * i / vdp2height);
+   glPixelZoom((float)outputwidth / (float)vdp2width, 0 - ((float)outputheight / (float)(vdp2height+i+i)));
    glDrawPixels(vdp2width, vdp2height, GL_RGBA, GL_UNSIGNED_BYTE, dispbuffer);
 
    if (! OSDUseBuffer())
@@ -3028,12 +3090,6 @@ void VIDSoftVdp2DrawEnd(void)
 #endif
 
    YuiSwapBuffers();
-
-   if ((Vdp1Regs->FBCR & 2) && (Vdp1Regs->TVMR & 8))
-   {
-      Vdp1External.manualerase = 1;
-      VIDSoftVdp1EraseFrameBuffer();
-   }
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -3042,6 +3098,13 @@ void VIDSoftVdp2DrawScreens(void)
 {
    int i;
 
+   VIDSoftVdp2SetResolution(Vdp2Regs->TVMD);
+   VIDSoftVdp2SetPriorityNBG0(Vdp2Regs->PRINA & 0x7);
+   VIDSoftVdp2SetPriorityNBG1((Vdp2Regs->PRINA >> 8) & 0x7);
+   VIDSoftVdp2SetPriorityNBG2(Vdp2Regs->PRINB & 0x7);
+   VIDSoftVdp2SetPriorityNBG3((Vdp2Regs->PRINB >> 8) & 0x7);
+   VIDSoftVdp2SetPriorityRBG0(Vdp2Regs->PRIR & 0x7);
+
    for (i = 7; i > 0; i--)
    {   
       if (nbg3priority == i)
@@ -3198,18 +3261,6 @@ void FASTCALL VIDSoftVdp2SetPriorityRBG0(int priority)
 
 //////////////////////////////////////////////////////////////////////////////
 
-void VIDSoftOnScreenDebugMessage(char *string, ...)
-{
-   va_list arglist;
-
-   va_start(arglist, string);
-   vsprintf(message, string, arglist);
-   va_end(arglist);
-   msglength = strlen(message);
-}
-
-//////////////////////////////////////////////////////////////////////////////
-
 void VIDSoftVdp1SwapFrameBuffer(void)
 {
    if (((Vdp1Regs->FBCR & 2) == 0) || Vdp1External.manualchange)
