@@ -24,7 +24,7 @@
 
 #define VIDCORE_SOFT   2
 
-extern u32 *dispbuffer;
+extern pixel_t *dispbuffer;
 
 extern VideoInterface_struct VIDSoft;
 
