@@ -25,6 +25,7 @@
 #include "ygl.h"
 #include "yui.h"
 #include "vidshared.h"
+#include "error.h"
 
 #ifdef WIN32
 #include <windows.h>
@@ -195,7 +196,7 @@ GLAPI void APIENTRY glActiveTexturedmy (GLenum texture){}
 #define IS_ZERO(a) ( (a) < EPS && (a) > -EPS)
 
 // AXB = |A||B|sin
-INLINE float cross2d( float veca[2], float vecb[2] )
+static INLINE float cross2d( float veca[2], float vecb[2] )
 {
    return (veca[0]*vecb[1])-(vecb[0]*veca[1]);
 }
@@ -207,9 +208,9 @@ INLINE float cross2d( float veca[2], float vecb[2] )
   a2+-+-----+b2
       ans
       
-  get intersection point for opssite edge.
+  get intersection point for opposite edge.
 --------------------------------------------*/  
-int FASTCALL YglIntersectionOppsiteEdge(float * a1, float * a2, float * b1, float * b2, float * out ) 
+int FASTCALL YglIntersectionOppositeEdge(float * a1, float * a2, float * b1, float * b2, float * out ) 
 {
   float veca[2];
   float vecb[2];
@@ -282,7 +283,7 @@ int YglCalcTextureQ(
    p4[1]=pnts[7];
 
    // calcurate Q1
-   if( YglIntersectionOppsiteEdge( p3, p1, p2, p4,  o ) == 0 )
+   if( YglIntersectionOppositeEdge( p3, p1, p2, p4,  o ) == 0 )
    {
       dx = o[0]-p1[0];
       if( !IS_ZERO(dx) )
@@ -312,7 +313,7 @@ int YglCalcTextureQ(
    /* q2 = 1.0f; */
 
    // calcurate Q3
-   if( YglIntersectionOppsiteEdge( p1, p3, p2,p4,  o ) == 0 )
+   if( YglIntersectionOppositeEdge( p1, p3, p2,p4,  o ) == 0 )
    {
       dx = o[0]-p3[0];
       if( !IS_ZERO(dx) )
@@ -341,7 +342,7 @@ int YglCalcTextureQ(
 
    
    // calcurate Q4
-   if( YglIntersectionOppsiteEdge( p3, p1, p4, p2,  o ) == 0 )
+   if( YglIntersectionOppositeEdge( p3, p1, p4, p2,  o ) == 0 )
    {
       dx = o[0]-p1[0];
       if( !IS_ZERO(dx) )
@@ -566,16 +567,6 @@ int YglGLInit(int width, int height) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-int YglScreenInit(int r, int g, int b, int d) {
-   YuiSetVideoAttribute(RED_SIZE, r);
-   YuiSetVideoAttribute(GREEN_SIZE, g);
-   YuiSetVideoAttribute(BLUE_SIZE, b);
-   YuiSetVideoAttribute(DEPTH_SIZE, d);
-   return (YuiSetVideoMode(320, 224, 32, 0) == 0);
-}
-
-//////////////////////////////////////////////////////////////////////////////
-
 
 int YglInit(int width, int height, unsigned int depth) {
    unsigned int i,j;
@@ -620,20 +611,6 @@ int YglInit(int width, int height, unsigned int depth) {
      }
    }
 
-   YuiSetVideoAttribute(DOUBLEBUFFER, 1);
-
-   if (!YglScreenInit(8, 8, 8, 24))
-   {
-      if (!YglScreenInit(4, 4, 4, 24))
-      {
-         if (!YglScreenInit(5, 6, 5, 16))
-         {
-       YuiErrorMsg("Couldn't set GL mode\n");
-            return -1;
-         }
-      }
-   }
-
    YglGLInit(width, height);
 
    // Set up Extention
@@ -764,14 +741,13 @@ int YglInit(int width, int height, unsigned int depth) {
    status = glCheckFramebufferStatus(GL_FRAMEBUFFER);
    if( status != GL_FRAMEBUFFER_COMPLETE )
    {
-      printf("YglInit: Framebuffer status = %08X\n", status );
+      YabErrorMsg("YglInit: Framebuffer status = %08X", status );
       return -1;
    }
    
    glBindFramebuffer(GL_FRAMEBUFFER, 0 );   
    
    _Ygl->st = 0;
-   _Ygl->msglength = 0;
 
    // This is probably wrong, but it'll have to do for now
    if ((cachelist = (cache_struct *)malloc(0x100000 / 8 * sizeof(cache_struct))) == NULL)
@@ -1796,7 +1772,6 @@ void YglReset(void) {
       level->prg[j].currentQuad = 0;
      }
    }
-   _Ygl->msglength = 0;
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -1817,17 +1792,6 @@ void YglChangeResolution(int w, int h) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void YglOnScreenDebugMessage(char *string, ...) {
-   va_list arglist;
-
-   va_start(arglist, string);
-   vsprintf(_Ygl->message, string, arglist);
-   va_end(arglist);
-   _Ygl->msglength = (int)strlen(_Ygl->message);
-}
-
-//////////////////////////////////////////////////////////////////////////////
-
 int YglIsCached(u32 addr, YglCache * c ) {
    int i = 0;
 
